#charset utf-8
#pike __REAL_VERSION__

// ----------------------------------------------------------------
// Daylight-saving and war time rules
//
// NOTE: this file is generated by mkrules.pike;
//       please do not edit manually /Mirar
// ----------------------------------------------------------------

// ----------------------------------------------------------------
// all rules are based on the gregorian calendar, so
// this is the gregorian rule:
// ----------------------------------------------------------------

protected array gregorian_yjd(int jd)
{
   int d=jd-1721426;

   int century=(4*d+3)/146097;
   int century_jd=(century*146097)/4;
   int century_day=d-century_jd;
   int century_year=(100*century_day+75)/36525;

   int y=century*100+century_year+1;

   return
   ({
      y,
      1721426+century_year*365+century_year/4+century_jd,
      (!(((y)%4) || (!((y)%100) && ((y)%400))))
   });
}

// ----------------------------------------------------------------
// Base "Timezone with rules" class
// ----------------------------------------------------------------

class TZRules
{
   constant is_timezone=1;
   constant is_dst_timezone=1;
   protected int offset_to_utc;
   string name;

   protected function(string:string) tzformat;
   protected array names;

   protected void create(int offset,string _name)
   {
      offset_to_utc=offset;
      name=_name;
      if (has_value(name, "/"))
      {
         names=name/"/";
         tzformat=lambda(string s)
                  {
                     if (s=="") return names[0]; else return names[1];
                  };
      }
      else
         tzformat=lambda(string s) { return sprintf(name,s); };
   }

// the Rule:
// which julian day does dst start and end this year?
   protected array(array(string|int)) jd_year_periods(int jd);

// is (midnight) this julian day dst?
   array tz_jd(int jd)
   {
      array(array(string|int)) a=jd_year_periods(jd);

      int i=0,n=sizeof(a)-1;
      while (i<n)
      {
         array b=a[i+1];
         if (jd<b[0]) break;
         if (jd==b[0] && -offset_to_utc+b[1]>=0) break;
         i++;
      }

      return ({offset_to_utc-a[i][2],tzformat(a[i][3])});
   }

// is this unixtime (utc) dst?
   array tz_ux(int ux)
   {
      int jd=2440588+ux/86400;
      array(array(string|int)) a=jd_year_periods(jd);

      int i=0,n=sizeof(a)-1;
      while (i<n)
      {
         array b=a[i+1];
         if (jd<b[0]-1) break;
         if (jd<b[0]+1 &&
             ux<(b[0]-2440588)*86400+b[1]) break;
         i++;
      }

      return ({offset_to_utc-a[i][2],tzformat(a[i][3])});
   }

   protected string _sprintf(int t)
  {
    return t=='O' && "Timezone("+name+")";
  }

   int raw_utc_offset() { return offset_to_utc; }
}

// ----------------------------------------------------------------------
// DST Rules
// ----------------------------------------------------------------------

// useful macros
#define FIXED(D)   (yjd+((D)-1))
#define FIX_L(D)   (yjd+leap+((D)-1))
#define LDAY(D,W)  (yjd+((D)-1)-( (yjd+((D)+(8-W)-1)) % 7))
#define LDAYL(D,W) (yjd+((D)-1)+leap-( (yjd+leap+((D)+(8-W)-1)) % 7))
#define UO offset_to_utc

// ----------------------------------------------------------------------
class Algeria
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(166)  ,UO+82800  ,3600 ,"S"   }),  // Jun 14
                     ({LDAY (281,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1917:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(83)   ,UO+82800  ,3600 ,"S"   }),  // Mar 24
                     ({LDAY (280,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(68)   ,UO+82800  ,3600 ,"S"   }),  // Mar 9
                     ({LDAY (280,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+82800  ,3600 ,"S"   }),  // Mar 1
                     ({LDAY (280,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(45)   ,UO+82800  ,3600 ,"S"   }),  // Feb 14
                     ({FIXED(297)  ,UO+82800  ,0    ,""    })});// Oct 23
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(73)   ,UO+82800  ,3600 ,"S"   }),  // Mar 14
                     ({FIXED(172)  ,UO+82800  ,0    ,""    })});// Jun 21
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(254)  ,UO+82800  ,3600 ,"S"   }),  // Sep 11
                     ({FIXED(323)  ,UO+0      ,0    ,""    })});// Nov 19
         case 1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (98,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(282)  ,UO+3600   ,0    ,""    })});// Oct 8
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(259)  ,UO+0      ,0    ,""    })});// Sep 16
         case 1971:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(115)  ,UO+82800  ,3600 ,"S"   }),  // Apr 25
                     ({FIXED(269)  ,UO+82800  ,0    ,""    })});// Sep 26
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(126)  ,UO+0      ,3600 ,"S"   }),  // May 6
                     ({FIXED(294)  ,UO-3600   ,0    ,""    })});// Oct 21
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(83)   ,UO+3600   ,3600 ,"S"   }),  // Mar 24
                     ({FIXED(265)  ,UO+7200   ,0    ,""    })});// Sep 22
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(116)  ,UO+0      ,3600 ,"S"   }),  // Apr 25
                     ({FIXED(305)  ,UO+3600   ,0    ,""    })});// Oct 31
         default: // ..1915 and 1981..
         case 1922..1938:
         case 1940..1943:
         case 1946..1970:
         case 1972..1976:
         case 1979:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Egypt
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(197)  ,UO+0      ,3600 ,"S"   }),  // Jul 15
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1941:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+0      ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(259)  ,UO-3600   ,0    ,""    })});// Sep 16
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(300)  ,UO-3600   ,0    ,""    })});// Oct 27
         case 1943..1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIX_L(305)  ,UO-3600   ,0    ,""    })});// Nov 1
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(106)  ,UO+0      ,3600 ,"S"   }),  // Apr 16
                     ({FIXED(305)  ,UO-3600   ,0    ,""    })});// Nov 1
         case 1957:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(130)  ,UO+0      ,3600 ,"S"   }),  // May 10
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1958:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1959..1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+3600   ,3600 ,"S"   }),  // May 1
                     ({FIX_L(273)  ,UO+7200   ,0    ,""    })});// Sep 30
         case 1982:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(206)  ,UO+3600   ,3600 ,"S"   }),  // Jul 25
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(193)  ,UO+3600   ,3600 ,"S"   }),  // Jul 12
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1989:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(126)  ,UO+3600   ,3600 ,"S"   }),  // May 6
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1966..1981:
         case 1984..1988:
         case 1990..1994:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+3600   ,3600 ,"S"   }),  // May 1
                     ({FIX_L(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1995..2005:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(120,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({LDAYL(273,4),UO+82800  ,0    ,""    })});// Sep lastThu
         case 2006:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (120,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({FIXED(264)  ,UO+82800  ,0    ,""    })});// Sep 21
         case 2007:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (120,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({LDAY (250,4),UO+82800  ,0    ,""    })});// Sep Thu>=1
         case 2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (121,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({LDAY (244,4),UO+82800  ,0    ,""    })});// Aug lastThu
         case 2009:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (120,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({FIXED(232)  ,UO+82800  ,0    ,""    })});// Aug 20
         case 2010:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (120,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({FIXED(222)  ,UO+82800  ,0    ,""    }),  // Aug 10
                     ({FIXED(252)  ,UO+86400  ,3600 ,"S"   }),  // Sep 9
                     ({LDAY (273,4),UO+82800  ,0    ,""    })});// Sep lastThu
         case 2014:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(135)  ,UO+86400  ,3600 ,"S"   }),  // May 15
                     ({FIXED(177)  ,UO+82800  ,0    ,""    }),  // Jun 26
                     ({FIXED(212)  ,UO+86400  ,3600 ,"S"   }),  // Jul 31
                     ({LDAY (273,4),UO+82800  ,0    ,""    })});// Sep lastThu
         default: // ..1939:
         case 1946..1956:
         case 2011..2013:
         case 2015..2022:
            return ({({0           ,0         ,0    ,""    })});
         case 2023..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(120,5),UO+0      ,3600 ,"S"   }),  // Apr lastFri
                     ({LDAYL(304,4),UO+82800  ,0    ,""    })});// Oct lastThu
      }
   }
}

class Libya
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1951:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(287)  ,UO+7200   ,3600 ,"S"   })});// Oct 14
         case 1953:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(282)  ,UO+7200   ,3600 ,"S"   })});// Oct 9
         case 1955:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(273)  ,UO+0      ,3600 ,"S"   })});// Sep 30
         case 1952:
         case 1954:
         case 1956:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(1)    ,UO-3600   ,0    ,""    })});// Jan 1
         case 1985:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+0      ,3600 ,"S"   }),  // Apr 6
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1986:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+0      ,3600 ,"S"   }),  // Apr 4
                     ({FIXED(276)  ,UO-3600   ,0    ,""    })});// Oct 3
         case 1982..1984:
         case 1987..1989:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1997:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+0      ,3600 ,"S"   }),  // Apr 4
                     ({FIXED(277)  ,UO-3600   ,0    ,""    })});// Oct 4
         case 2013:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,5) ,UO+3600   ,3600 ,"S"   }),  // Mar lastFri
                     ({LDAY (304,5),UO+3600   ,0    ,""    })});// Oct lastFri
         default: // ..1950 and 2014..
         case 1957..1981:
         case 1990..1996:
         case 1998..2012:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Mauritius
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1982:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(283)  ,UO+0      ,3600 ,""    })});// Oct 10
         case 1983:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(80)   ,UO-3600   ,0    ,""    })});// Mar 21
         case 2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (305,7),UO+7200   ,3600 ,""    })});// Oct lastSun
         case 2009:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (90,7) ,UO+3600   ,0    ,""    })});// Mar lastSun
         default: // ..1981 and 2010..
         case 1984..2007:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Morocco
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(255)  ,UO+0      ,3600 ,""    }),  // Sep 12
                     ({FIXED(323)  ,UO-3600   ,0    ,""    })});// Nov 19
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(56)   ,UO+0      ,3600 ,""    })});// Feb 25
         case 1941..1944:
            return ({({0           ,0         ,3600 ,""    })});
         case 1945:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(322)  ,UO-3600   ,0    ,""    })});// Nov 18
         case 1950:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(162)  ,UO+0      ,3600 ,""    }),  // Jun 11
                     ({FIXED(302)  ,UO-3600   ,0    ,""    })});// Oct 29
         case 1967:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(154)  ,UO+43200  ,3600 ,""    }),  // Jun 3
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(175)  ,UO+0      ,3600 ,""    }),  // Jun 24
                     ({FIXED(244)  ,UO-3600   ,0    ,""    })});// Sep 1
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+0      ,3600 ,""    }),  // May 1
                     ({FIXED(214)  ,UO-3600   ,0    ,""    })});// Aug 1
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,""    }),  // May 1
                     ({FIXED(271)  ,UO-3600   ,0    ,""    })});// Sep 28
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,""    }),  // Jun 1
                     ({FIXED(216)  ,UO-3600   ,0    ,""    })});// Aug 4
         case 2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,""    }),  // Jun 1
                     ({FIXED(245)  ,UO-3600   ,0    ,""    })});// Sep 1
         case 2009:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,""    }),  // Jun 1
                     ({FIXED(233)  ,UO-3600   ,0    ,""    })});// Aug 21
         case 2010:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+0      ,3600 ,""    }),  // May 2
                     ({FIXED(220)  ,UO-3600   ,0    ,""    })});// Aug 8
         case 2011:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+0      ,3600 ,""    }),  // Apr 3
                     ({FIXED(212)  ,UO-3600   ,0    ,""    })});// Jul 31
         case 2012:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (121,7),UO+7200   ,3600 ,""    }),  // Apr lastSun
                     ({FIXED(202)  ,UO+7200   ,0    ,""    }),  // Jul 20
                     ({FIXED(233)  ,UO+7200   ,3600 ,""    }),  // Aug 20
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Sep 30
         case 2013:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,""    }),  // Apr lastSun
                     ({FIXED(188)  ,UO+7200   ,0    ,""    }),  // Jul 7
                     ({FIXED(222)  ,UO+7200   ,3600 ,""    }),  // Aug 10
                     ({LDAY (304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 2014:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({FIXED(179)  ,UO+7200   ,0    ,""    }),  // Jun 28
                     ({FIXED(214)  ,UO+7200   ,3600 ,""    }),  // Aug 2
                     ({LDAY (304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 2015:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({FIXED(165)  ,UO+7200   ,0    ,""    }),  // Jun 14
                     ({FIXED(200)  ,UO+7200   ,3600 ,""    }),  // Jul 19
                     ({LDAY (304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 2016:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (91,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({FIXED(157)  ,UO+7200   ,0    ,""    }),  // Jun 5
                     ({FIXED(192)  ,UO+7200   ,3600 ,""    }),  // Jul 10
                     ({LDAY (305,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 2017:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({FIXED(141)  ,UO+7200   ,0    ,""    }),  // May 21
                     ({FIXED(183)  ,UO+7200   ,3600 ,""    }),  // Jul 2
                     ({LDAY (304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 2018:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({FIXED(133)  ,UO+7200   ,0    ,""    }),  // May 13
                     ({FIXED(168)  ,UO+7200   ,3600 ,""    }),  // Jun 17
                     ({LDAY (304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 2019:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(125)  ,UO+10800  ,-3600,""    }),  // May 5
                     ({FIXED(160)  ,UO+10800  ,0    ,""    })});// Jun 9
         case 2020:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(110)  ,UO+10800  ,-3600,""    }),  // Apr 19
                     ({FIXED(152)  ,UO+10800  ,0    ,""    })});// May 31
         case 2021:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(101)  ,UO+10800  ,-3600,""    }),  // Apr 11
                     ({FIXED(136)  ,UO+10800  ,0    ,""    })});// May 16
         case 2022:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(86)   ,UO+10800  ,-3600,""    }),  // Mar 27
                     ({FIXED(128)  ,UO+10800  ,0    ,""    })});// May 8
         case 2023:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(78)   ,UO+10800  ,-3600,""    }),  // Mar 19
                     ({FIXED(113)  ,UO+10800  ,0    ,""    })});// Apr 23
         case 2024:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(70)   ,UO+10800  ,-3600,""    }),  // Mar 10
                     ({FIXED(105)  ,UO+10800  ,0    ,""    })});// Apr 14
         case 2025:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(54)   ,UO+10800  ,-3600,""    }),  // Feb 23
                     ({FIXED(96)   ,UO+10800  ,0    ,""    })});// Apr 6
         case 2026:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(46)   ,UO+10800  ,-3600,""    }),  // Feb 15
                     ({FIXED(81)   ,UO+10800  ,0    ,""    })});// Mar 22
         case 2027:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(38)   ,UO+10800  ,-3600,""    }),  // Feb 7
                     ({FIXED(73)   ,UO+10800  ,0    ,""    })});// Mar 14
         case 2028:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(23)   ,UO+10800  ,-3600,""    }),  // Jan 23
                     ({FIXED(65)   ,UO+10800  ,0    ,""    })});// Mar 5
         case 2029:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(14)   ,UO+10800  ,-3600,""    }),  // Jan 14
                     ({FIXED(49)   ,UO+10800  ,0    ,""    }),  // Feb 18
                     ({FIXED(364)  ,UO+10800  ,-3600,""    })});// Dec 30
         case 2030:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(41)   ,UO+10800  ,0    ,""    }),  // Feb 10
                     ({FIXED(356)  ,UO+10800  ,-3600,""    })});// Dec 22
         case 2031:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(26)   ,UO+10800  ,0    ,""    }),  // Jan 26
                     ({FIXED(348)  ,UO+10800  ,-3600,""    })});// Dec 14
         case 2032:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(18)   ,UO+10800  ,0    ,""    }),  // Jan 18
                     ({FIXED(333)  ,UO+10800  ,-3600,""    })});// Nov 28
         case 2033:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(9)    ,UO+10800  ,0    ,""    }),  // Jan 9
                     ({FIXED(324)  ,UO+10800  ,-3600,""    }),  // Nov 20
                     ({FIXED(359)  ,UO+10800  ,0    ,""    })});// Dec 25
         case 2034:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(309)  ,UO+10800  ,-3600,""    }),  // Nov 5
                     ({FIXED(351)  ,UO+10800  ,0    ,""    })});// Dec 17
         case 2035:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(301)  ,UO+10800  ,-3600,""    }),  // Oct 28
                     ({FIXED(343)  ,UO+10800  ,0    ,""    })});// Dec 9
         case 2036:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(293)  ,UO+10800  ,-3600,""    }),  // Oct 19
                     ({FIXED(328)  ,UO+10800  ,0    ,""    })});// Nov 23
         case 2037:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(277)  ,UO+10800  ,-3600,""    }),  // Oct 4
                     ({FIXED(319)  ,UO+10800  ,0    ,""    })});// Nov 15
         case 2038:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(269)  ,UO+10800  ,-3600,""    }),  // Sep 26
                     ({FIXED(304)  ,UO+10800  ,0    ,""    })});// Oct 31
         case 2039:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(261)  ,UO+10800  ,-3600,""    }),  // Sep 18
                     ({FIXED(296)  ,UO+10800  ,0    ,""    })});// Oct 23
         case 2040:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(246)  ,UO+10800  ,-3600,""    }),  // Sep 2
                     ({FIXED(288)  ,UO+10800  ,0    ,""    })});// Oct 14
         case 2041:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(237)  ,UO+10800  ,-3600,""    }),  // Aug 25
                     ({FIXED(272)  ,UO+10800  ,0    ,""    })});// Sep 29
         case 2042:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(222)  ,UO+10800  ,-3600,""    }),  // Aug 10
                     ({FIXED(264)  ,UO+10800  ,0    ,""    })});// Sep 21
         case 2043:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(214)  ,UO+10800  ,-3600,""    }),  // Aug 2
                     ({FIXED(256)  ,UO+10800  ,0    ,""    })});// Sep 13
         case 2044:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(206)  ,UO+10800  ,-3600,""    }),  // Jul 24
                     ({FIXED(241)  ,UO+10800  ,0    ,""    })});// Aug 28
         case 2045:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(190)  ,UO+10800  ,-3600,""    }),  // Jul 9
                     ({FIXED(232)  ,UO+10800  ,0    ,""    })});// Aug 20
         case 2046:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(182)  ,UO+10800  ,-3600,""    }),  // Jul 1
                     ({FIXED(217)  ,UO+10800  ,0    ,""    })});// Aug 5
         case 2047:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(174)  ,UO+10800  ,-3600,""    }),  // Jun 23
                     ({FIXED(209)  ,UO+10800  ,0    ,""    })});// Jul 28
         case 2048:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(159)  ,UO+10800  ,-3600,""    }),  // Jun 7
                     ({FIXED(201)  ,UO+10800  ,0    ,""    })});// Jul 19
         case 2049:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(150)  ,UO+10800  ,-3600,""    }),  // May 30
                     ({FIXED(185)  ,UO+10800  ,0    ,""    })});// Jul 4
         case 2050:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(135)  ,UO+10800  ,-3600,""    }),  // May 15
                     ({FIXED(177)  ,UO+10800  ,0    ,""    })});// Jun 26
         case 2051:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(127)  ,UO+10800  ,-3600,""    }),  // May 7
                     ({FIXED(169)  ,UO+10800  ,0    ,""    })});// Jun 18
         case 2052:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(119)  ,UO+10800  ,-3600,""    }),  // Apr 28
                     ({FIXED(154)  ,UO+10800  ,0    ,""    })});// Jun 2
         case 2053:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(103)  ,UO+10800  ,-3600,""    }),  // Apr 13
                     ({FIXED(145)  ,UO+10800  ,0    ,""    })});// May 25
         case 2054:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(95)   ,UO+10800  ,-3600,""    }),  // Apr 5
                     ({FIXED(130)  ,UO+10800  ,0    ,""    })});// May 10
         case 2055:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(87)   ,UO+10800  ,-3600,""    }),  // Mar 28
                     ({FIXED(122)  ,UO+10800  ,0    ,""    })});// May 2
         case 2056:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(72)   ,UO+10800  ,-3600,""    }),  // Mar 12
                     ({FIXED(114)  ,UO+10800  ,0    ,""    })});// Apr 23
         case 2057:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(63)   ,UO+10800  ,-3600,""    }),  // Mar 4
                     ({FIXED(98)   ,UO+10800  ,0    ,""    })});// Apr 8
         case 2058:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(48)   ,UO+10800  ,-3600,""    }),  // Feb 17
                     ({FIXED(90)   ,UO+10800  ,0    ,""    })});// Mar 31
         case 2059:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(40)   ,UO+10800  ,-3600,""    }),  // Feb 9
                     ({FIXED(82)   ,UO+10800  ,0    ,""    })});// Mar 23
         case 2060:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(32)   ,UO+10800  ,-3600,""    }),  // Feb 1
                     ({FIXED(67)   ,UO+10800  ,0    ,""    })});// Mar 7
         case 2061:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(16)   ,UO+10800  ,-3600,""    }),  // Jan 16
                     ({FIXED(58)   ,UO+10800  ,0    ,""    })});// Feb 27
         case 2062:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(8)    ,UO+10800  ,-3600,""    }),  // Jan 8
                     ({FIXED(43)   ,UO+10800  ,0    ,""    }),  // Feb 12
                     ({FIXED(365)  ,UO+10800  ,-3600,""    })});// Dec 31
         case 2063:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(35)   ,UO+10800  ,0    ,""    }),  // Feb 4
                     ({FIXED(350)  ,UO+10800  ,-3600,""    })});// Dec 16
         case 2064:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(27)   ,UO+10800  ,0    ,""    }),  // Jan 27
                     ({FIXED(342)  ,UO+10800  ,-3600,""    })});// Dec 7
         case 2065:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(11)   ,UO+10800  ,0    ,""    }),  // Jan 11
                     ({FIXED(326)  ,UO+10800  ,-3600,""    })});// Nov 22
         case 2066:
            return ({({0           ,0         ,-3600,""    }),  
                     ({FIXED(3)    ,UO+10800  ,0    ,""    }),  // Jan 3
                     ({FIXED(318)  ,UO+10800  ,-3600,""    }),  // Nov 14
                     ({FIXED(360)  ,UO+10800  ,0    ,""    })});// Dec 26
         case 2067:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(310)  ,UO+10800  ,-3600,""    }),  // Nov 6
                     ({FIXED(345)  ,UO+10800  ,0    ,""    })});// Dec 11
         case 2068:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(295)  ,UO+10800  ,-3600,""    }),  // Oct 21
                     ({FIXED(337)  ,UO+10800  ,0    ,""    })});// Dec 2
         case 2069:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(286)  ,UO+10800  ,-3600,""    }),  // Oct 13
                     ({FIXED(321)  ,UO+10800  ,0    ,""    })});// Nov 17
         case 2070:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(278)  ,UO+10800  ,-3600,""    }),  // Oct 5
                     ({FIXED(313)  ,UO+10800  ,0    ,""    })});// Nov 9
         case 2071:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(263)  ,UO+10800  ,-3600,""    }),  // Sep 20
                     ({FIXED(305)  ,UO+10800  ,0    ,""    })});// Nov 1
         case 2072:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(255)  ,UO+10800  ,-3600,""    }),  // Sep 11
                     ({FIXED(290)  ,UO+10800  ,0    ,""    })});// Oct 16
         case 2073:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(239)  ,UO+10800  ,-3600,""    }),  // Aug 27
                     ({FIXED(281)  ,UO+10800  ,0    ,""    })});// Oct 8
         case 2074:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(231)  ,UO+10800  ,-3600,""    }),  // Aug 19
                     ({FIXED(273)  ,UO+10800  ,0    ,""    })});// Sep 30
         case 2075:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(223)  ,UO+10800  ,-3600,""    }),  // Aug 11
                     ({FIXED(258)  ,UO+10800  ,0    ,""    })});// Sep 15
         case 2076:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(208)  ,UO+10800  ,-3600,""    }),  // Jul 26
                     ({FIXED(250)  ,UO+10800  ,0    ,""    })});// Sep 6
         case 2077:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(199)  ,UO+10800  ,-3600,""    }),  // Jul 18
                     ({FIXED(234)  ,UO+10800  ,0    ,""    })});// Aug 22
         case 2078:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(191)  ,UO+10800  ,-3600,""    }),  // Jul 10
                     ({FIXED(226)  ,UO+10800  ,0    ,""    })});// Aug 14
         case 2079:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(176)  ,UO+10800  ,-3600,""    }),  // Jun 25
                     ({FIXED(218)  ,UO+10800  ,0    ,""    })});// Aug 6
         case 2080:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(168)  ,UO+10800  ,-3600,""    }),  // Jun 16
                     ({FIXED(203)  ,UO+10800  ,0    ,""    })});// Jul 21
         case 2081:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+10800  ,-3600,""    }),  // Jun 1
                     ({FIXED(194)  ,UO+10800  ,0    ,""    })});// Jul 13
         case 2082:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(144)  ,UO+10800  ,-3600,""    }),  // May 24
                     ({FIXED(179)  ,UO+10800  ,0    ,""    })});// Jun 28
         case 2083:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(136)  ,UO+10800  ,-3600,""    }),  // May 16
                     ({FIXED(171)  ,UO+10800  ,0    ,""    })});// Jun 20
         case 2084:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+10800  ,-3600,""    }),  // Apr 30
                     ({FIXED(163)  ,UO+10800  ,0    ,""    })});// Jun 11
         case 2085:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(112)  ,UO+10800  ,-3600,""    }),  // Apr 22
                     ({FIXED(147)  ,UO+10800  ,0    ,""    })});// May 27
         case 2086:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(104)  ,UO+10800  ,-3600,""    }),  // Apr 14
                     ({FIXED(139)  ,UO+10800  ,0    ,""    })});// May 19
         case 2087:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+10800  ,-3600,""    }),  // Mar 30
                     ({FIXED(131)  ,UO+10800  ,0    ,""    })});// May 11
         default: // ..1938 and 2088..
         case 1946..1949:
         case 1951..1966:
         case 1968..1973:
         case 1975:
         case 1979..2007:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Namibia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1994:
            return ({({0           ,0         ,0    ,"CAT" }),  // ?
                     ({FIXED(80)   ,UO+0      ,-3600,"WAT" }),  // Mar 21
                     ({LDAY (250,7),UO+10800  ,0    ,"CAT" })});// Sep Sun>=1
         case 1995..2017:
            return ({({0           ,0         ,0    ,"CAT" }),  
                     ({LDAYL(97,7) ,UO+7200   ,-3600,"WAT" }),  // Apr Sun>=1
                     ({LDAYL(250,7),UO+10800  ,0    ,"CAT" })});// Sep Sun>=1
         default: // ..1993 and 2018..
            return ({({0           ,0         ,0    ,"CAT" })});
      }
   }
}

class SA
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (264,7),UO+7200   ,3600 ,""    })});// Sep Sun>=15
         case 1943:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (80,7) ,UO+3600   ,0    ,""    }),  // Mar Sun>=15
                     ({LDAY (264,7),UO+7200   ,3600 ,""    })});// Sep Sun>=15
         case 1944:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (81,7) ,UO+3600   ,0    ,""    })});// Mar Sun>=15
         default: // ..1941 and 1945..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Sudan
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1970:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(288)  ,UO-3600   ,0    ,""    })});// Oct 15
         case 1971:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"S"   }),  // Apr 30
                     ({FIXED(288)  ,UO-3600   ,0    ,""    })});// Oct 15
         case 1972..1985:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(120,7),UO+0      ,3600 ,"S"   }),  // Apr lastSun
                     ({FIX_L(288)  ,UO-3600   ,0    ,""    })});// Oct 15
         default: // ..1969 and 1986..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Tunisia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(322)  ,UO+82800  ,0    ,""    })});// Nov 18
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(56)   ,UO+82800  ,3600 ,"S"   })});// Feb 25
         case 1941:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(279)  ,UO-3600   ,0    ,""    })});// Oct 6
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(68)   ,UO+0      ,3600 ,"S"   }),  // Mar 9
                     ({FIXED(306)  ,UO+7200   ,0    ,""    })});// Nov 2
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+7200   ,3600 ,"S"   }),  // Mar 29
                     ({FIXED(107)  ,UO+3600   ,0    ,""    }),  // Apr 17
                     ({FIXED(115)  ,UO+7200   ,3600 ,"S"   }),  // Apr 25
                     ({FIXED(277)  ,UO+3600   ,0    ,""    })});// Oct 4
         case 1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (98,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(282)  ,UO-3600   ,0    ,""    })});// Oct 8
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(259)  ,UO-3600   ,0    ,""    })});// Sep 16
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"S"   }),  // Apr 30
                     ({FIXED(267)  ,UO+0      ,0    ,""    })});// Sep 24
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(274)  ,UO+0      ,0    ,""    })});// Oct 1
         case 1988:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({LDAY (274,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1989:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+0      ,3600 ,"S"   }),  // Mar 26
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1990:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 2005:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(273)  ,UO+3600   ,0    ,""    })});// Sep 30
         case 2006..2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         default: // ..1938 and 2009..
         case 1946..1976:
         case 1979..1987:
         case 1991..2004:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Troll
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..2003:
            return ({({0           ,0         ,0    ,"+00" })});// ?
         case 2004:
            return ({({0           ,0         ,0    ,"+00" }),  // ?
                     ({LDAY (305,7),3600      ,0    ,"+00" })});// Oct lastSun
         case 2005..:
            return ({({0           ,0         ,0    ,"+00" }),  
                     ({LDAYL(90,7) ,3600      ,7200 ,"+02" }),  // Mar lastSun
                     ({LDAYL(304,7),3600      ,0    ,"+00" })});// Oct lastSun
      }
   }
}

class EUAsia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1978:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(273,7),3600      ,0    ,""    })});// Sep lastSun
         case 1981..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,3600      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),3600      ,0    ,""    })});// Sep lastSun
         case 1996..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,3600      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),3600      ,0    ,""    })});// Oct lastSun
      }
   }
}

class E_EurAsia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1978:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1981..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1996..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(304,7),UO-3600   ,0    ,""    })});// Oct lastSun
      }
   }
}

class RussiaAsia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1981..1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,""    }),  // Apr 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1984:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,""    }),  // Apr 1
                     ({LDAY (274,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1985..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1996..2010:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         default: // ..1980 and 2011..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Armenia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 2011:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (90,7) ,UO+7200   ,3600 ,""    }),  // Mar lastSun
                     ({LDAY (304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         default: // ..2010 and 2012..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Azer
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1997..2015:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+14400  ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(304,7),UO+14400  ,0    ,""    })});// Oct lastSun
         default: // ..1996 and 2016..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Dhaka
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 2009:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(170)  ,UO+82800  ,3600 ,""    }),  // Jun 19
                     ({FIXED(365)  ,UO+82800  ,0    ,""    })});// Dec 31
         default: // ..2008 and 2010..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Shang
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1919:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(102)  ,UO+86400  ,3600 ,"D"   }),  // Apr 12
                     ({FIXED(273)  ,UO+82800  ,0    ,"S"   })});// Sep 30
         case 1940:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,"D"   }),  // Jun 1
                     ({FIXED(286)  ,UO+82800  ,0    ,"S"   })});// Oct 12
         case 1941:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(74)   ,UO+0      ,3600 ,"D"   }),  // Mar 15
                     ({FIXED(305)  ,UO+82800  ,0    ,"S"   })});// Nov 1
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(31)   ,UO+0      ,3600 ,"D"   })});// Jan 31
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"D"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(244)  ,UO+82800  ,0    ,"S"   })});// Sep 1
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(135)  ,UO+0      ,3600 ,"D"   }),  // May 15
                     ({FIXED(273)  ,UO+82800  ,0    ,"S"   })});// Sep 30
         case 1947:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(105)  ,UO+0      ,3600 ,"D"   }),  // Apr 15
                     ({FIXED(304)  ,UO+82800  ,0    ,"S"   })});// Oct 31
         case 1948..1949:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIX_L(121)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({FIX_L(273)  ,UO+82800  ,0    ,"S"   })});// Sep 30
         default: // ..1918 and 1950..
         case 1920..1939:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class PRC
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1986:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(124)  ,UO+7200   ,3600 ,"D"   }),  // May 4
                     ({LDAY (260,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=11
         case 1987..1991:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(107,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=11
                     ({LDAYL(260,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=11
         default: // ..1985 and 1992..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class HK
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(111)  ,UO+0      ,3600 ,"S"   }),  // Apr 21
                     ({FIXED(335)  ,UO+12600  ,0    ,""    })});// Dec 1
         case 1947:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(103)  ,UO+12600  ,3600 ,"S"   }),  // Apr 13
                     ({FIXED(334)  ,UO+12600  ,0    ,""    })});// Nov 30
         case 1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(123)  ,UO+12600  ,3600 ,"S"   }),  // May 2
                     ({LDAY (308,7),UO+12600  ,0    ,""    })});// Oct Sun>=28
         case 1949..1952:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+12600  ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(307,7),UO+12600  ,0    ,""    })});// Oct Sun>=28
         case 1953:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+12600  ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAY (310,7),UO+9000   ,0    ,""    })});// Oct Sun>=31
         case 1954..1964:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(83,7) ,UO+12600  ,3600 ,"S"   }),  // Mar Sun>=18
                     ({LDAYL(310,7),UO+9000   ,0    ,""    })});// Oct Sun>=31
         case 1973:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (112,7),UO+12600  ,3600 ,"S"   }),  // Apr Sun>=16
                     ({LDAY (295,7),UO+9000   ,0    ,""    }),  // Oct Sun>=16
                     ({FIXED(364)  ,UO+12600  ,3600 ,"S"   })});// Dec 30
         case 1974:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({LDAY (112,7),UO+9000   ,3600 ,"S"   }),  // Apr Sun>=16
                     ({LDAY (295,7),UO+9000   ,0    ,""    })});// Oct Sun>=16
         case 1965..1972:
         case 1975..1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(112,7),UO+12600  ,3600 ,"S"   }),  // Apr Sun>=16
                     ({LDAYL(295,7),UO+9000   ,0    ,""    })});// Oct Sun>=16
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(133)  ,UO+12600  ,3600 ,"S"   }),  // May 13
                     ({FIXED(294)  ,UO+9000   ,0    ,""    })});// Oct 21
         default: // ..1945 and 1980..
         case 1977..1978:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Taiwan
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(135)  ,UO+0      ,3600 ,"D"   }),  // May 15
                     ({FIXED(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         case 1947:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(105)  ,UO+0      ,3600 ,"D"   }),  // Apr 15
                     ({FIXED(305)  ,UO-3600   ,0    ,"S"   })});// Nov 1
         case 1948..1951:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIX_L(121)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         case 1952:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(61)   ,UO+0      ,3600 ,"D"   }),  // Mar 1
                     ({FIXED(306)  ,UO-3600   ,0    ,"S"   })});// Nov 1
         case 1953..1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"D"   }),  // Apr 1
                     ({FIXED(305)  ,UO-3600   ,0    ,"S"   })});// Nov 1
         case 1960..1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIX_L(152)  ,UO+0      ,3600 ,"D"   }),  // Jun 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         case 1955..1959:
         case 1974..1975:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIX_L(91)   ,UO+0      ,3600 ,"D"   }),  // Apr 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         case 1979:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(182)  ,UO+0      ,3600 ,"D"   }),  // Jul 1
                     ({FIXED(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         default: // ..1945 and 1980..
         case 1962..1973:
         case 1976..1978:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Macau
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1941:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(120)  ,UO+82800  ,3600 ,""    }),  // Apr 30
                     ({FIXED(321)  ,UO+79200  ,0    ,""    })});// Nov 17
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(120)  ,UO+82800  ,3600 ,""    }),  // Apr 30
                     ({FIXED(273)  ,UO+79200  ,0    ,"S"   })});// Sep 30
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,UO+82800  ,3600 ,"D"   }),  // Apr 30
                     ({FIXED(273)  ,UO+82800  ,0    ,"S"   })});// Sep 30
         case 1947:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(109)  ,UO+82800  ,3600 ,"D"   }),  // Apr 19
                     ({FIXED(334)  ,UO+82800  ,0    ,"S"   })});// Nov 30
         case 1948:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(123)  ,UO+82800  ,3600 ,"D"   }),  // May 2
                     ({FIXED(305)  ,UO+82800  ,0    ,"S"   })});// Oct 31
         case 1951:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(90)   ,UO+82800  ,3600 ,"D"   }),  // Mar 31
                     ({FIXED(301)  ,UO+82800  ,0    ,"S"   })});// Oct 28
         case 1952:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (98,6) ,UO+82800  ,3600 ,"D"   }),  // Apr Sat>=1
                     ({FIXED(306)  ,UO+82800  ,0    ,"S"   })});// Nov 1
         case 1949..1950:
         case 1953:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (97,6) ,UO+82800  ,3600 ,"D"   }),  // Apr Sat>=1
                     ({LDAY (304,6),UO+82800  ,0    ,"S"   })});// Oct lastSat
         case 1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (82,6) ,UO+82800  ,3600 ,"D"   }),  // Mar Sat>=17
                     ({LDAY (304,6),UO+82800  ,0    ,"S"   })});// Oct lastSat
         case 1955:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (82,6) ,UO+82800  ,3600 ,"D"   }),  // Mar Sat>=17
                     ({FIXED(309)  ,UO+82800  ,0    ,"S"   })});// Nov 5
         case 1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (83,6) ,UO+82800  ,3600 ,"D"   }),  // Mar Sat>=17
                     ({LDAY (312,7),UO+9000   ,0    ,"S"   })});// Nov Sun>=1
         case 1957..1964:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(83,7) ,UO+12600  ,3600 ,"D"   }),  // Mar Sun>=18
                     ({LDAYL(311,7),UO+9000   ,0    ,"S"   })});// Nov Sun>=1
         case 1965..1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (112,7),UO+12600  ,3600 ,"D"   }),  // Apr Sun>=16
                     ({LDAY (295,7),UO+5400   ,0    ,"S"   })});// Oct Sun>=16
         case 1973:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (112,7),UO+12600  ,3600 ,"D"   }),  // Apr Sun>=16
                     ({LDAY (295,7),UO+9000   ,0    ,"S"   }),  // Oct Sun>=16
                     ({FIXED(364)  ,UO+12600  ,3600 ,"D"   })});// Dec 30
         case 1974:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (295,7),UO+9000   ,0    ,"S"   })});// Oct Sun>=16
         case 1967..1972:
         case 1975..1976:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(112,7),UO+12600  ,3600 ,"D"   }),  // Apr Sun>=16
                     ({LDAYL(295,7),UO+9000   ,0    ,"S"   })});// Oct Sun>=16
         case 1979:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(133)  ,UO+12600  ,3600 ,"D"   }),  // May 13
                     ({LDAY (295,7),UO+9000   ,0    ,"S"   })});// Oct Sun>=16
         case 1944..1945:
         case 1977..1978:
         case 1980..:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Cyprus
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1974:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1975:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(103)  ,UO+0      ,3600 ,"S"   }),  // Apr 13
                     ({FIXED(285)  ,UO-3600   ,0    ,""    })});// Oct 12
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(136)  ,UO+0      ,3600 ,"S"   }),  // May 15
                     ({FIXED(285)  ,UO-3600   ,0    ,""    })});// Oct 11
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(268)  ,UO-3600   ,0    ,""    })});// Sep 25
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 2
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1981..1997:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1998:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+0      ,3600 ,"S"   })});// Mar lastSun
         case 1999..:
            return ({({0           ,0         ,3600 ,"S"   })});
      }
   }
}

class Iran
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1910:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(1)    ,UO+0      ,0    ,""    })});// Jan 1
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(80)   ,UO+82800  ,3600 ,""    }),  // Mar 21
                     ({FIXED(293)  ,UO+82800  ,0    ,""    })});// Oct 20
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(83)   ,UO+86400  ,3600 ,""    }),  // Mar 24
                     ({FIXED(217)  ,UO+0      ,0    ,""    })});// Aug 5
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(146)  ,UO+86400  ,3600 ,""    }),  // May 26
                     ({FIXED(261)  ,UO+82800  ,0    ,""    })});// Sep 18
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(80)   ,UO+86400  ,3600 ,""    }),  // Mar 20
                     ({FIXED(266)  ,UO+82800  ,0    ,""    })});// Sep 22
         case 1991:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+86400  ,3600 ,""    }),  // May 2
                     ({FIXED(264)  ,UO+82800  ,0    ,""    })});// Sep 21
         case 1996:
         case 2000:
         case 2004:
         case 2008:
         case 2012:
         case 2016:
         case 2020:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(80)   ,UO+86400  ,3600 ,""    }),  // Mar 20
                     ({FIXED(264)  ,UO+82800  ,0    ,""    })});// Sep 20
         case 1992..1995:
         case 1997..1999:
         case 2001..2003:
         case 2005:
         case 2009..2011:
         case 2013..2015:
         case 2017..2019:
         case 2021..2022:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(80)   ,UO+86400  ,3600 ,""    }),  // Mar 21
                     ({FIX_L(264)  ,UO+82800  ,0    ,""    })});// Sep 21
         default: // ..1909 and 2023..
         case 1911..1976:
         case 1981..1990:
         case 2006..2007:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Iraq
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1982:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(121)  ,UO+0      ,3600 ,""    }),  // May 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(90)   ,UO+0      ,3600 ,""    }),  // Mar 31
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1984:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,""    }),  // Apr 1
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1985:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,""    }),  // Apr 1
                     ({LDAY (273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1986..1990:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+3600   ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1991..2007:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(91)   ,UO+10800  ,3600 ,""    }),  // Apr 1
                     ({FIX_L(274)  ,UO+10800  ,0    ,""    })});// Oct 1
         default: // ..1981 and 2008..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Zion
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1940:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(152)  ,86400     ,3600 ,"D"   }),  // May 31
                     ({FIXED(274)  ,86400     ,0    ,"S"   }),  // Sep 30
                     ({FIXED(321)  ,86400     ,3600 ,"D"   })});// Nov 16
         case 1941:
            return ({({0           ,0         ,3600 ,"D"   })});
         case 1942:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(304)  ,86400     ,0    ,"S"   })});// Oct 31
         case 1943..1944:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIX_L(90)   ,86400     ,3600 ,"D"   }),  // Mar 31
                     ({FIX_L(304)  ,86400     ,0    ,"S"   })});// Oct 31
         case 1945..1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(105)  ,86400     ,3600 ,"D"   }),  // Apr 15
                     ({FIXED(304)  ,86400     ,0    ,"S"   })});// Oct 31
         case 1948:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(143)  ,86400     ,7200 ,"DD"  }),  // May 22
                     ({FIXED(244)  ,86400     ,3600 ,"D"   }),  // Aug 31
                     ({FIXED(305)  ,86400     ,0    ,"S"   })});// Oct 31
         case 1949:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,86400     ,3600 ,"D"   }),  // Apr 30
                     ({FIXED(304)  ,86400     ,0    ,"S"   })});// Oct 31
         case 1950:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(105)  ,86400     ,3600 ,"D"   }),  // Apr 15
                     ({FIXED(257)  ,86400     ,0    ,"S"   })});// Sep 14
         case 1951:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(90)   ,86400     ,3600 ,"D"   }),  // Mar 31
                     ({FIXED(314)  ,86400     ,0    ,"S"   })});// Nov 10
         case 1952:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(110)  ,86400     ,3600 ,"D"   }),  // Apr 19
                     ({FIXED(292)  ,86400     ,0    ,"S"   })});// Oct 18
         case 1953:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(101)  ,86400     ,3600 ,"D"   }),  // Apr 11
                     ({FIXED(255)  ,86400     ,0    ,"S"   })});// Sep 12
         case 1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(163)  ,86400     ,3600 ,"D"   }),  // Jun 12
                     ({FIXED(254)  ,86400     ,0    ,"S"   })});// Sep 11
         case 1955:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(162)  ,86400     ,3600 ,"D"   }),  // Jun 11
                     ({FIXED(253)  ,86400     ,0    ,"S"   })});// Sep 10
         case 1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(154)  ,86400     ,3600 ,"D"   }),  // Jun 2
                     ({FIXED(273)  ,86400     ,0    ,"S"   })});// Sep 29
         case 1957:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(117)  ,86400     ,3600 ,"D"   }),  // Apr 27
                     ({FIXED(264)  ,86400     ,0    ,"S"   })});// Sep 21
         case 1974:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(187)  ,UO+86400  ,3600 ,"D"   }),  // Jul 6
                     ({FIXED(285)  ,UO+82800  ,0    ,"S"   })});// Oct 12
         case 1975:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(109)  ,UO+86400  ,3600 ,"D"   }),  // Apr 19
                     ({FIXED(242)  ,UO+82800  ,0    ,"S"   })});// Aug 30
         case 1980:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(215)  ,UO+86400  ,3600 ,"D"   }),  // Aug 2
                     ({FIXED(257)  ,UO+86400  ,0    ,"S"   })});// Sep 13
         default: // ..1939:
         case 1947:
         case 1958..1973:
         case 1976..1979:
         case 1981..1983:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1984:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(126)  ,UO+86400  ,3600 ,"D"   }),  // May 5
                     ({FIXED(238)  ,UO+86400  ,0    ,"S"   })});// Aug 25
         case 1985:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(103)  ,UO+86400  ,3600 ,"D"   }),  // Apr 13
                     ({FIXED(243)  ,UO+82800  ,0    ,"S"   })});// Aug 31
         case 1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(137)  ,UO+86400  ,3600 ,"D"   }),  // May 17
                     ({FIXED(249)  ,UO+82800  ,0    ,"S"   })});// Sep 6
         case 1987:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(104)  ,UO+86400  ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(255)  ,UO+82800  ,0    ,"S"   })});// Sep 12
         case 1988:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(100)  ,UO+86400  ,3600 ,"D"   }),  // Apr 9
                     ({FIXED(247)  ,UO+82800  ,0    ,"S"   })});// Sep 3
         case 1989:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(119)  ,UO+86400  ,3600 ,"D"   }),  // Apr 29
                     ({FIXED(245)  ,UO+82800  ,0    ,"S"   })});// Sep 2
         case 1990:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(83)   ,UO+86400  ,3600 ,"D"   }),  // Mar 24
                     ({FIXED(237)  ,UO+82800  ,0    ,"S"   })});// Aug 25
         case 1991:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(82)   ,UO+86400  ,3600 ,"D"   }),  // Mar 23
                     ({FIXED(243)  ,UO+82800  ,0    ,"S"   })});// Aug 31
         case 1992:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(88)   ,UO+86400  ,3600 ,"D"   }),  // Mar 28
                     ({FIXED(249)  ,UO+82800  ,0    ,"S"   })});// Sep 5
         case 1993:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,"D"   }),  // Apr 2
                     ({FIXED(248)  ,UO-3600   ,0    ,"S"   })});// Sep 5
         case 1994:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"D"   }),  // Apr 1
                     ({FIXED(240)  ,UO-3600   ,0    ,"S"   })});// Aug 28
         case 1995:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(90)   ,UO+0      ,3600 ,"D"   }),  // Mar 31
                     ({FIXED(246)  ,UO-3600   ,0    ,"S"   })});// Sep 3
         case 1996:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(74)   ,UO+86400  ,3600 ,"D"   }),  // Mar 14
                     ({FIXED(259)  ,UO+82800  ,0    ,"S"   })});// Sep 15
         case 1997:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(79)   ,UO+86400  ,3600 ,"D"   }),  // Mar 20
                     ({FIXED(256)  ,UO+82800  ,0    ,"S"   })});// Sep 13
         case 1998:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(79)   ,UO+0      ,3600 ,"D"   }),  // Mar 20
                     ({FIXED(249)  ,UO-3600   ,0    ,"S"   })});// Sep 6
         case 1999:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"D"   }),  // Apr 2
                     ({FIXED(246)  ,UO+3600   ,0    ,"S"   })});// Sep 3
         case 2000:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(105)  ,UO+7200   ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(280)  ,UO+0      ,0    ,"S"   })});// Oct 6
         case 2001:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(99)   ,UO+3600   ,3600 ,"D"   }),  // Apr 9
                     ({FIXED(267)  ,UO+0      ,0    ,"S"   })});// Sep 24
         case 2002:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(88)   ,UO+3600   ,3600 ,"D"   }),  // Mar 29
                     ({FIXED(280)  ,UO+0      ,0    ,"S"   })});// Oct 7
         case 2003:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(87)   ,UO+3600   ,3600 ,"D"   }),  // Mar 28
                     ({FIXED(276)  ,UO+0      ,0    ,"S"   })});// Oct 3
         case 2004:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(98)   ,UO+3600   ,3600 ,"D"   }),  // Apr 7
                     ({FIXED(266)  ,UO+0      ,0    ,"S"   })});// Sep 22
         case 2005:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(282)  ,UO+3600   ,0    ,"S"   })});// Oct 9
         case 2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(274)  ,UO+3600   ,0    ,"S"   })});// Oct 1
         case 2007:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(259)  ,UO+3600   ,0    ,"S"   })});// Sep 16
         case 2008:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (92,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(279)  ,UO+3600   ,0    ,"S"   })});// Oct 5
         case 2009:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(270)  ,UO+3600   ,0    ,"S"   })});// Sep 27
         case 2010:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(255)  ,UO+3600   ,0    ,"S"   })});// Sep 12
         case 2011:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(275)  ,UO+3600   ,0    ,"S"   })});// Oct 2
         case 2012:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (92,5) ,UO+7200   ,3600 ,"D"   }),  // Apr Fri<=1
                     ({FIXED(267)  ,UO+3600   ,0    ,"S"   })});// Sep 23
         case 2013..:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(88,5) ,UO+7200   ,3600 ,"D"   }),  // Mar Fri>=23
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
      }
   }
}

class Japan
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1949:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (97,6) ,UO+86400  ,3600 ,"D"   }),  // Apr Sat>=1
                     ({LDAY (257,6),UO+86400  ,0    ,"S"   })});// Sep Sat>=8
         case 1948:
         case 1950..1951:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,6),UO+86400  ,3600 ,"D"   }),  // May Sat>=1
                     ({LDAYL(257,6),UO+86400  ,0    ,"S"   })});// Sep Sat>=8
         default: // ..1947 and 1952..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Jordan
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1973:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(157)  ,UO+0      ,3600 ,"S"   }),  // Jun 6
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(306)  ,UO-3600   ,0    ,""    })});// Nov 1
         case 1974..1975:
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"S"   }),  // Apr 30
                     ({FIXED(273)  ,UO-3600   ,0    ,""    })});// Sep 30
         case 1985:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1989:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(128)  ,UO+0      ,3600 ,"S"   }),  // May 8
                     ({LDAY (280,5),UO-3600   ,0    ,""    })});// Oct Fri>=1
         case 1990:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(117)  ,UO+0      ,3600 ,"S"   }),  // Apr 27
                     ({LDAY (280,5),UO-3600   ,0    ,""    })});// Oct Fri>=1
         case 1991:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+0      ,3600 ,"S"   }),  // Apr 17
                     ({FIXED(270)  ,UO-3600   ,0    ,""    })});// Sep 27
         case 1992:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(101)  ,UO+0      ,3600 ,"S"   }),  // Apr 10
                     ({LDAY (281,5),UO-3600   ,0    ,""    })});// Oct Fri>=1
         case 1986..1988:
         case 1993:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,5) ,UO+0      ,3600 ,"S"   }),  // Apr Fri>=1
                     ({LDAYL(280,5),UO-3600   ,0    ,""    })});// Oct Fri>=1
         case 1994:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,5) ,UO+0      ,3600 ,"S"   }),  // Apr Fri>=1
                     ({LDAY (264,5),UO-3600   ,0    ,""    })});// Sep Fri>=15
         case 1995..1998:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,5) ,UO+0      ,3600 ,"S"   }),  // Apr Fri>=1
                     ({LDAYL(264,5),UO+0      ,0    ,""    })});// Sep Fri>=15
         case 1999:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(182)  ,UO+0      ,3600 ,"S"   }),  // Jul 1
                     ({LDAY (273,5),UO+0      ,0    ,""    })});// Sep lastFri
         case 2000..2001:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,4) ,UO+0      ,3600 ,"S"   }),  // Mar lastThu
                     ({LDAYL(273,5),UO+0      ,0    ,""    })});// Sep lastFri
         case 2003:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({FIXED(297)  ,UO+0      ,0    ,""    })});// Oct 24
         case 2004:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (91,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({FIXED(289)  ,UO+0      ,0    ,""    })});// Oct 15
         case 2002:
         case 2005:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({LDAY (273,5),UO+0      ,0    ,""    })});// Sep lastFri
         case 2012:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (91,4) ,UO+86400  ,3600 ,"S"   })});// Mar lastThu
         case 2013:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(354)  ,UO-3600   ,0    ,""    })});// Dec 20
         case 2006..2011:
         case 2014..2021:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({LDAYL(304,5),UO+0      ,0    ,""    })});// Oct lastFri
         case 2022:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (59,4) ,UO+86400  ,3600 ,"S"   }),  // Feb lastThu
                     ({LDAY (304,5),UO+0      ,0    ,""    })});// Oct lastFri
         default: // ..1972 and 2023..
         case 1979..1984:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Kyrgyz
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1991:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1992..1996:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(103,7),UO+0      ,3600 ,""    }),  // Apr Sun>=7
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1997..2004:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+9000   ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(304,7),UO+5400   ,0    ,""    })});// Oct lastSun
         case 2005:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+9000   ,3600 ,""    })});// Mar lastSun
         case 2006..:
            return ({({0           ,0         ,3600 ,""    })});
      }
   }
}

class ROK
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1948:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(153)  ,UO+0      ,3600 ,"D"   }),  // Jun 1
                     ({FIXED(256)  ,UO+82800  ,0    ,"S"   })});// Sep 12
         case 1949:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(93)   ,UO+0      ,3600 ,"D"   }),  // Apr 3
                     ({LDAY (256,6),UO+82800  ,0    ,"S"   })});// Sep Sat>=7
         case 1950:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"D"   }),  // Apr 1
                     ({LDAY (256,6),UO+82800  ,0    ,"S"   })});// Sep Sat>=7
         case 1951:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(126)  ,UO+0      ,3600 ,"D"   }),  // May 6
                     ({LDAY (256,6),UO+82800  ,0    ,"S"   })});// Sep Sat>=7
         case 1955:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(125)  ,UO+0      ,3600 ,"D"   }),  // May 5
                     ({FIXED(251)  ,UO+82800  ,0    ,"S"   })});// Sep 8
         case 1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(141)  ,UO+0      ,3600 ,"D"   }),  // May 20
                     ({FIXED(273)  ,UO+82800  ,0    ,"S"   })});// Sep 29
         case 1957..1960:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(266,6),UO+82800  ,0    ,"S"   })});// Sep Sat>=17
         case 1987..1988:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(134,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=8
                     ({LDAYL(287,7),UO+7200   ,0    ,"S"   })});// Oct Sun>=8
         default: // ..1947 and 1989..
         case 1952..1954:
         case 1961..1986:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Lebanon
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(88)   ,UO+0      ,3600 ,"S"   }),  // Mar 28
                     ({FIXED(299)  ,UO-3600   ,0    ,""    })});// Oct 25
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+0      ,3600 ,"S"   }),  // Apr 3
                     ({FIXED(276)  ,UO-3600   ,0    ,""    })});// Oct 3
         case 1922:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+0      ,3600 ,"S"   }),  // Mar 26
                     ({FIXED(281)  ,UO-3600   ,0    ,""    })});// Oct 8
         case 1923:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(112)  ,UO+0      ,3600 ,"S"   }),  // Apr 22
                     ({FIXED(259)  ,UO-3600   ,0    ,""    })});// Sep 16
         case 1972:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(174)  ,UO+0      ,3600 ,"S"   }),  // Jun 22
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1957..1961:
         case 1973..1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"S"   }),  // Apr 30
                     ({FIXED(273)  ,UO-3600   ,0    ,""    })});// Sep 30
         default: // ..1919:
         case 1924..1956:
         case 1962..1971:
         case 1979..1983:
            return ({({0           ,0         ,0    ,""    })});
         case 1988:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({FIXED(290)  ,UO-3600   ,0    ,""    })});// Oct 16
         case 1989:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(130)  ,UO+0      ,3600 ,"S"   }),  // May 10
                     ({FIXED(289)  ,UO-3600   ,0    ,""    })});// Oct 16
         case 1984..1987:
         case 1990..1991:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIX_L(289)  ,UO-3600   ,0    ,""    })});// Oct 16
         case 1992:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(278)  ,UO-3600   ,0    ,""    })});// Oct 4
         case 1993..1998:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1999..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO-3600   ,0    ,""    })});// Oct lastSun
      }
   }
}

class NBorneo
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1935..1941:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(257)  ,UO+0      ,1200 ,""    }),  // Sep 14
                     ({FIX_L(348)  ,UO-1200   ,0    ,""    })});// Dec 14
         default: // ..1934 and 1942..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Mongol
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(91)   ,UO+0      ,3600 ,""    }),  // Apr 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1984:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,""    }),  // Apr 1
                     ({LDAY (274,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1985..1998:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,""    }),  // Mar lastSun
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 2001:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (120,6),UO+7200   ,3600 ,""    }),  // Apr lastSat
                     ({LDAY (273,6),UO+3600   ,0    ,""    })});// Sep lastSat
         case 2002..2006:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,6) ,UO+7200   ,3600 ,""    }),  // Mar lastSat
                     ({LDAYL(273,6),UO+3600   ,0    ,""    })});// Sep lastSat
         case 2015..2016:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,6) ,UO+7200   ,3600 ,""    }),  // Mar lastSat
                     ({LDAYL(273,6),UO-3600   ,0    ,""    })});// Sep lastSat
         default: // ..1982 and 2017..
         case 1999..2000:
         case 2007..2014:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Pakistan
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 2002:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (98,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=2
                     ({LDAY (281,7),UO-3600   ,0    ,""    })});// Oct Sun>=2
         case 2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({FIXED(306)  ,UO-3600   ,0    ,""    })});// Nov 1
         case 2009:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+0      ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(305)  ,UO-3600   ,0    ,""    })});// Nov 1
         default: // ..2001 and 2010..
         case 2003..2007:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class EgyptAsia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1956:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1957:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(130)  ,UO+0      ,3600 ,"S"   }),  // May 10
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1958:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1959..1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+3600   ,3600 ,"S"   }),  // May 1
                     ({FIX_L(273)  ,UO+7200   ,0    ,""    })});// Sep 30
         case 1966:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+3600   ,3600 ,"S"   }),  // May 1
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1967:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+3600   ,3600 ,"S"   })});// May 1
         case 1968..:
            return ({({0           ,0         ,3600 ,"S"   })});
      }
   }
}

class Palestine
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1998:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1999..2003:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(111,5),UO+0      ,3600 ,"S"   }),  // Apr Fri>=15
                     ({LDAYL(294,5),UO-3600   ,0    ,""    })});// Oct Fri>=15
         case 2004:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (112,5),UO+0      ,3600 ,"S"   }),  // Apr Fri>=15
                     ({FIXED(275)  ,UO+0      ,0    ,""    })});// Oct 1
         case 2005:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (111,5),UO+0      ,3600 ,"S"   }),  // Apr Fri>=15
                     ({FIXED(277)  ,UO+3600   ,0    ,""    })});// Oct 4
         case 2006:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(265)  ,UO-3600   ,0    ,""    })});// Sep 22
         case 2007:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(256)  ,UO+3600   ,0    ,""    })});// Sep 13
         case 2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (91,5) ,UO+0      ,3600 ,"S"   }),  // Mar lastFri
                     ({FIXED(245)  ,UO-3600   ,0    ,""    })});// Sep 1
         case 2009:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,5) ,UO+0      ,3600 ,"S"   }),  // Mar lastFri
                     ({FIXED(247)  ,UO+0      ,0    ,""    })});// Sep 4
         case 2010:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+0      ,3600 ,"S"   }),  // Mar 26
                     ({FIXED(223)  ,UO-3600   ,0    ,""    })});// Aug 11
         case 2011:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+60     ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(213)  ,UO-3600   ,0    ,""    }),  // Aug 1
                     ({FIXED(242)  ,UO+0      ,3600 ,"S"   }),  // Aug 30
                     ({FIXED(273)  ,UO-3600   ,0    ,""    })});// Sep 30
         case 2012:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (91,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({FIXED(265)  ,UO+0      ,0    ,""    })});// Sep 21
         case 2013:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({FIXED(270)  ,UO-3600   ,0    ,""    })});// Sep 27
         case 2014:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,4) ,UO+86400  ,3600 ,"S"   }),  // Mar lastThu
                     ({FIXED(297)  ,UO-3600   ,0    ,""    })});// Oct 24
         case 2015:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(87)   ,UO+0      ,3600 ,"S"   }),  // Mar 28
                     ({FIXED(296)  ,UO+0      ,0    ,""    })});// Oct 23
         case 2016..2018:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(89,6) ,UO+3600   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({LDAYL(303,6),UO+0      ,0    ,""    })});// Oct Sat<=30
         case 2019:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+0      ,3600 ,"S"   }),  // Mar 29
                     ({LDAY (303,6),UO-3600   ,0    ,""    })});// Oct Sat<=30
         case 2020:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+0      ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(298)  ,UO+0      ,0    ,""    })});// Oct 24
         case 2021:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+0      ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(302)  ,UO+0      ,0    ,""    })});// Oct 29
         case 2022:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(86)   ,UO+0      ,3600 ,"S"   }),  // Mar 27
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2024:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(111)  ,UO+7200   ,3600 ,"S"   }),  // Apr 20
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2025:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(102)  ,UO+7200   ,3600 ,"S"   }),  // Apr 12
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2036:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(292)  ,UO+3600   ,0    ,""    })});// Oct 18
         case 2037:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(283)  ,UO+3600   ,0    ,""    })});// Oct 10
         case 2038:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(268)  ,UO+3600   ,0    ,""    })});// Sep 25
         case 2039:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(260)  ,UO+3600   ,0    ,""    })});// Sep 17
         case 2040:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(245)  ,UO+3600   ,0    ,""    }),  // Sep 1
                     ({FIXED(294)  ,UO+7200   ,3600 ,"S"   }),  // Oct 20
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2041:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(236)  ,UO+3600   ,0    ,""    }),  // Aug 24
                     ({FIXED(278)  ,UO+7200   ,3600 ,"S"   }),  // Oct 5
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2042:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(228)  ,UO+3600   ,0    ,""    }),  // Aug 16
                     ({FIXED(270)  ,UO+7200   ,3600 ,"S"   }),  // Sep 27
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2043:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(213)  ,UO+3600   ,0    ,""    }),  // Aug 1
                     ({FIXED(262)  ,UO+7200   ,3600 ,"S"   }),  // Sep 19
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2044:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(205)  ,UO+3600   ,0    ,""    }),  // Jul 23
                     ({FIXED(247)  ,UO+7200   ,3600 ,"S"   }),  // Sep 3
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2045:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(196)  ,UO+3600   ,0    ,""    }),  // Jul 15
                     ({FIXED(238)  ,UO+7200   ,3600 ,"S"   }),  // Aug 26
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2046:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(181)  ,UO+3600   ,0    ,""    }),  // Jun 30
                     ({FIXED(230)  ,UO+7200   ,3600 ,"S"   }),  // Aug 18
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2047:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(173)  ,UO+3600   ,0    ,""    }),  // Jun 22
                     ({FIXED(215)  ,UO+7200   ,3600 ,"S"   }),  // Aug 3
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2048:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(158)  ,UO+3600   ,0    ,""    }),  // Jun 6
                     ({FIXED(207)  ,UO+7200   ,3600 ,"S"   }),  // Jul 25
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2049:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(149)  ,UO+3600   ,0    ,""    }),  // May 29
                     ({FIXED(191)  ,UO+7200   ,3600 ,"S"   }),  // Jul 10
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2050:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(141)  ,UO+3600   ,0    ,""    }),  // May 21
                     ({FIXED(183)  ,UO+7200   ,3600 ,"S"   }),  // Jul 2
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2051:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(126)  ,UO+3600   ,0    ,""    }),  // May 6
                     ({FIXED(175)  ,UO+7200   ,3600 ,"S"   }),  // Jun 24
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2052:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(118)  ,UO+3600   ,0    ,""    }),  // Apr 27
                     ({FIXED(160)  ,UO+7200   ,3600 ,"S"   }),  // Jun 8
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2053:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(102)  ,UO+3600   ,0    ,""    }),  // Apr 12
                     ({FIXED(151)  ,UO+7200   ,3600 ,"S"   }),  // May 31
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2054:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(94)   ,UO+3600   ,0    ,""    }),  // Apr 4
                     ({FIXED(143)  ,UO+7200   ,3600 ,"S"   }),  // May 23
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2055:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(128)  ,UO+7200   ,3600 ,"S"   }),  // May 8
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2023:
         case 2056:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(119)  ,UO+7200   ,3600 ,"S"   }),  // Apr 29
                     ({LDAYL(303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2057:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(104)  ,UO+7200   ,3600 ,"S"   }),  // Apr 14
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2058:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+7200   ,3600 ,"S"   }),  // Apr 6
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2068:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(294)  ,UO+3600   ,0    ,""    })});// Oct 20
         case 2069:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(285)  ,UO+3600   ,0    ,""    })});// Oct 12
         case 2070:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(277)  ,UO+3600   ,0    ,""    })});// Oct 4
         case 2071:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(262)  ,UO+3600   ,0    ,""    })});// Sep 19
         case 2072:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(254)  ,UO+3600   ,0    ,""    }),  // Sep 10
                     ({FIXED(296)  ,UO+7200   ,3600 ,"S"   }),  // Oct 22
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2073:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(245)  ,UO+3600   ,0    ,""    }),  // Sep 2
                     ({FIXED(287)  ,UO+7200   ,3600 ,"S"   }),  // Oct 14
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2074:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(230)  ,UO+3600   ,0    ,""    }),  // Aug 18
                     ({FIXED(279)  ,UO+7200   ,3600 ,"S"   }),  // Oct 6
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2075:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(222)  ,UO+3600   ,0    ,""    }),  // Aug 10
                     ({FIXED(264)  ,UO+7200   ,3600 ,"S"   }),  // Sep 21
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2076:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(207)  ,UO+3600   ,0    ,""    }),  // Jul 25
                     ({FIXED(256)  ,UO+7200   ,3600 ,"S"   }),  // Sep 12
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2077:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(198)  ,UO+3600   ,0    ,""    }),  // Jul 17
                     ({FIXED(247)  ,UO+7200   ,3600 ,"S"   }),  // Sep 4
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2078:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(190)  ,UO+3600   ,0    ,""    }),  // Jul 9
                     ({FIXED(232)  ,UO+7200   ,3600 ,"S"   }),  // Aug 20
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2079:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(175)  ,UO+3600   ,0    ,""    }),  // Jun 24
                     ({FIXED(224)  ,UO+7200   ,3600 ,"S"   }),  // Aug 12
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2080:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(167)  ,UO+3600   ,0    ,""    }),  // Jun 15
                     ({FIXED(209)  ,UO+7200   ,3600 ,"S"   }),  // Jul 27
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2081:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(158)  ,UO+3600   ,0    ,""    }),  // Jun 7
                     ({FIXED(200)  ,UO+7200   ,3600 ,"S"   }),  // Jul 19
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2082:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(143)  ,UO+3600   ,0    ,""    }),  // May 23
                     ({FIXED(192)  ,UO+7200   ,3600 ,"S"   }),  // Jul 11
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2083:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(135)  ,UO+3600   ,0    ,""    }),  // May 15
                     ({FIXED(177)  ,UO+7200   ,3600 ,"S"   }),  // Jun 26
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2084:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(120)  ,UO+3600   ,0    ,""    }),  // Apr 29
                     ({FIXED(169)  ,UO+7200   ,3600 ,"S"   }),  // Jun 17
                     ({LDAY (304,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2085:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(111)  ,UO+3600   ,0    ,""    }),  // Apr 21
                     ({FIXED(160)  ,UO+7200   ,3600 ,"S"   }),  // Jun 9
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2086:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({FIXED(103)  ,UO+3600   ,0    ,""    }),  // Apr 13
                     ({FIXED(145)  ,UO+7200   ,3600 ,"S"   }),  // May 25
                     ({LDAY (303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
         case 2026..2035:
         case 2059..2067:
         case 2087..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(89,6) ,UO+7200   ,3600 ,"S"   }),  // Mar Sat<=30
                     ({LDAYL(303,6),UO+3600   ,0    ,""    })});// Oct Sat<=30
      }
   }
}

class Phil
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1936:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(306)  ,UO+0      ,3600 ,"D"   })});// Nov 1
         case 1937:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(32)   ,UO-3600   ,0    ,"S"   })});// Feb 1
         case 1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(102)  ,UO+0      ,3600 ,"D"   }),  // Apr 12
                     ({FIXED(182)  ,UO-3600   ,0    ,"S"   })});// Jul 1
         case 1978:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(81)   ,UO+0      ,3600 ,"D"   }),  // Mar 22
                     ({FIXED(264)  ,UO-3600   ,0    ,"S"   })});// Sep 21
         default: // ..1935 and 1979..
         case 1938..1953:
         case 1955..1977:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Syria
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920..1923:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(111,7),UO+7200   ,3600 ,"S"   }),  // Apr Sun>=15
                     ({LDAYL(280,7),UO+3600   ,0    ,""    })});// Oct Sun>=1
         case 1962:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(119)  ,UO+7200   ,3600 ,"S"   }),  // Apr 29
                     ({FIXED(274)  ,UO+3600   ,0    ,""    })});// Oct 1
         case 1963:
         case 1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+7200   ,3600 ,"S"   }),  // May 1
                     ({FIXED(273)  ,UO+3600   ,0    ,""    })});// Sep 30
         case 1966:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(114)  ,UO+7200   ,3600 ,"S"   }),  // Apr 24
                     ({FIXED(274)  ,UO+3600   ,0    ,""    })});// Oct 1
         case 1964:
         case 1967..1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+7200   ,3600 ,"S"   }),  // May 1
                     ({FIX_L(274)  ,UO+3600   ,0    ,""    })});// Oct 1
         case 1977..1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+7200   ,3600 ,"S"   }),  // May 1
                     ({FIXED(244)  ,UO+3600   ,0    ,""    })});// Sep 1
         case 1983..1984:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(99)   ,UO+7200   ,3600 ,"S"   }),  // Apr 9
                     ({FIX_L(274)  ,UO+3600   ,0    ,""    })});// Oct 1
         case 1986:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(47)   ,UO+7200   ,3600 ,"S"   }),  // Feb 16
                     ({FIXED(282)  ,UO+3600   ,0    ,""    })});// Oct 9
         case 1987:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+7200   ,3600 ,"S"   }),  // Mar 1
                     ({FIXED(304)  ,UO+3600   ,0    ,""    })});// Oct 31
         case 1988:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(75)   ,UO+7200   ,3600 ,"S"   }),  // Mar 15
                     ({FIXED(305)  ,UO+3600   ,0    ,""    })});// Oct 31
         case 1989:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(90)   ,UO+7200   ,3600 ,"S"   }),  // Mar 31
                     ({FIXED(274)  ,UO+3600   ,0    ,""    })});// Oct 1
         case 1990:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+7200   ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(273)  ,UO+3600   ,0    ,""    })});// Sep 30
         case 1992:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+0      ,3600 ,"S"   }),  // Apr 8
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1993:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+0      ,3600 ,"S"   }),  // Mar 26
                     ({FIXED(268)  ,UO-3600   ,0    ,""    })});// Sep 25
         case 1997..1998:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,1) ,UO+0      ,3600 ,"S"   }),  // Mar lastMon
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1991:
         case 1994..1996:
         case 1999..2005:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 2006:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(265)  ,UO-3600   ,0    ,""    })});// Sep 22
         case 2007:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,5) ,UO+0      ,3600 ,"S"   }),  // Mar lastFri
                     ({LDAY (311,5),UO-3600   ,0    ,""    })});// Nov Fri>=1
         case 2008:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (98,5) ,UO+0      ,3600 ,"S"   }),  // Apr Fri>=1
                     ({FIXED(306)  ,UO-3600   ,0    ,""    })});// Nov 1
         case 2010..2011:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,5) ,UO+0      ,3600 ,"S"   }),  // Apr Fri>=1
                     ({LDAY (304,5),UO-3600   ,0    ,""    })});// Oct lastFri
         case 2009:
         case 2012..2022:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,5) ,UO+0      ,3600 ,"S"   }),  // Mar lastFri
                     ({LDAYL(304,5),UO-3600   ,0    ,""    })});// Oct lastFri
         default: // ..1919 and 2023..
         case 1924..1961:
         case 1979..1982:
         case 1985:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Aus
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1917:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(1)    ,UO+7200   ,3600 ,"D"   }),  // Jan 1
                     ({LDAY (90,7) ,UO+7200   ,0    ,"S"   })});// Mar lastSun
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(1)    ,UO+7200   ,3600 ,"D"   }),  // Jan 1
                     ({LDAY (90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({FIXED(270)  ,UO+7200   ,3600 ,"D"   })});// Sep 27
         case 1943:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({FIXED(276)  ,UO+7200   ,3600 ,"D"   })});// Oct 3
         case 1944:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (91,7) ,UO+7200   ,0    ,"S"   })});// Mar lastSun
         default: // ..1916 and 1945..
         case 1918..1941:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class AW
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1974:
         case 1983:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1991:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(321)  ,UO+7200   ,3600 ,"D"   })});// Nov 17
         case 1975:
         case 1984:
         case 1992:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(66,7) ,UO+7200   ,0    ,"S"   })});// Mar Sun>=1
         case 2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(337)  ,UO+7200   ,3600 ,"D"   })});// Dec 3
         case 2007..2008:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2009:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (90,7) ,UO+7200   ,0    ,"S"   })});// Mar lastSun
         default: // ..1973 and 2010..
         case 1976..1982:
         case 1985..1990:
         case 1993..2005:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class AQ
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1972:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (60,7) ,UO+7200   ,0    ,"S"   })});// Feb lastSun
         case 1971:
         case 1989:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1990..1991:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1992:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (67,7) ,UO+7200   ,0    ,"S"   })});// Mar Sun>=1
         default: // ..1970 and 1993..
         case 1973..1988:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Holiday
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1992:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1993:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1994:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   })});// Mar Sun>=1
         default: // ..1991 and 1995..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class AS
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1970:
            return ({({0           ,0         ,0    ,"S"   })});// ?
         case 1971:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1972:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(58)   ,UO+7200   ,0    ,"S"   }),  // Feb 27
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1973..1985:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1986:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({FIXED(292)  ,UO+7200   ,3600 ,"D"   })});// Oct 19
         case 1987..1990:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1991:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(62)   ,UO+7200   ,0    ,"S"   }),  // Mar 3
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1992:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(82)   ,UO+7200   ,0    ,"S"   }),  // Mar 22
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1993:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(66)   ,UO+7200   ,0    ,"S"   }),  // Mar 7
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1994:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(79)   ,UO+7200   ,0    ,"S"   }),  // Mar 20
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2006:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(92)   ,UO+7200   ,0    ,"S"   }),  // Apr 2
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1995..2005:
         case 2007:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2008..:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
      }
   }
}

class AT
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1917:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAY (301,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=22
         case 1918:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAY (301,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=22
         case 1919:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   })});// Mar Sun>=1
         default: // ..1915:
         case 1920..1966:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1916:
         case 1967:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
         case 1968:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (95,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=29
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1969..1971:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (73,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=8
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1972:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (60,7) ,UO+7200   ,0    ,"S"   }),  // Feb lastSun
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1982..1983:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1973..1981:
         case 1984..1985:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1986:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAY (294,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=15
         case 1987:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAY (301,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=22
         case 1988..1990:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2000:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (91,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAY (244,7),UO+7200   ,3600 ,"D"   })});// Aug lastSun
         case 1991..1999:
         case 2001..2005:
         case 2007:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
         case 2006:
         case 2008..:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
      }
   }
}

class AV
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1970:
            return ({({0           ,0         ,0    ,"S"   })});// ?
         case 1971:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1972:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (60,7) ,UO+7200   ,0    ,"S"   }),  // Feb lastSun
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1986..1987:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAY (294,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=15
         case 1988..1990:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1973..1985:
         case 1991..1994:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2000:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (91,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAY (244,7),UO+7200   ,3600 ,"D"   })});// Aug lastSun
         case 2006:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1995..1999:
         case 2001..2005:
         case 2007:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2008..:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
      }
   }
}

class AN
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1970:
            return ({({0           ,0         ,0    ,"S"   })});// ?
         case 1971:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1972:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(58)   ,UO+7200   ,0    ,"S"   }),  // Feb 27
                     ({LDAY (305,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1986:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({FIXED(292)  ,UO+7200   ,3600 ,"D"   })});// Oct 19
         case 1987..1989:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1973..1981:
         case 1983..1985:
         case 1990..1995:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2000:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (91,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAY (244,7),UO+7200   ,3600 ,"D"   })});// Aug lastSun
         case 1982:
         case 2006:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1996..1999:
         case 2001..2005:
         case 2007:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(90,7) ,UO+7200   ,0    ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 2008..:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
      }
   }
}

class LH
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1980:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1981:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (304,7),UO+7200   ,3600 ,""    })});// Oct lastSun
         case 1982..1984:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(66,7) ,UO+3600   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,3600 ,""    })});// Oct lastSun
         case 1985:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO+3600   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAY (304,7),UO+7200   ,1800 ,""    })});// Oct lastSun
         case 1986:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (80,7) ,UO+5400   ,0    ,""    }),  // Mar Sun>=15
                     ({FIXED(292)  ,UO+7200   ,1800 ,""    })});// Oct 19
         case 1987..1989:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAYL(80,7) ,UO+5400   ,0    ,""    }),  // Mar Sun>=15
                     ({LDAYL(304,7),UO+7200   ,1800 ,""    })});// Oct lastSun
         case 1990..1995:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAYL(66,7) ,UO+5400   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,1800 ,""    })});// Oct lastSun
         case 2000:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (91,7) ,UO+5400   ,0    ,""    }),  // Mar lastSun
                     ({LDAY (244,7),UO+7200   ,1800 ,""    })});// Aug lastSun
         case 2006:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (97,7) ,UO+5400   ,0    ,""    }),  // Apr Sun>=1
                     ({LDAY (304,7),UO+7200   ,1800 ,""    })});// Oct lastSun
         case 1996..1999:
         case 2001..2005:
         case 2007:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAYL(90,7) ,UO+5400   ,0    ,""    }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,1800 ,""    })});// Oct lastSun
         case 2008..:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAYL(97,7) ,UO+5400   ,0    ,""    }),  // Apr Sun>=1
                     ({LDAYL(280,7),UO+7200   ,1800 ,""    })});// Oct Sun>=1
      }
   }
}

class Fiji
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1998:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (311,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 1999:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (59,7) ,UO+7200   ,0    ,""    }),  // Feb lastSun
                     ({LDAY (311,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 2000:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (60,7) ,UO+7200   ,0    ,""    })});// Feb lastSun
         case 2009:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(333)  ,UO+7200   ,3600 ,""    })});// Nov 29
         case 2010:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,0    ,""    }),  // Mar lastSun
                     ({LDAY (300,7),UO+7200   ,3600 ,""    })});// Oct Sun>=21
         case 2011:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAY (300,7),UO+7200   ,3600 ,""    })});// Oct Sun>=21
         case 2012..2013:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (24,7) ,UO+7200   ,0    ,""    }),  // Jan Sun>=18
                     ({LDAYL(300,7),UO+7200   ,3600 ,""    })});// Oct Sun>=21
         case 2014:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (24,7) ,UO+3600   ,0    ,""    }),  // Jan Sun>=18
                     ({LDAY (311,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 2015..2018:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (18,7) ,UO+7200   ,0    ,""    }),  // Jan Sun>=12
                     ({LDAYL(311,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 2019:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (18,7) ,UO+7200   ,0    ,""    }),  // Jan Sun>=12
                     ({LDAY (318,7),UO+7200   ,3600 ,""    })});// Nov Sun>=8
         case 2020:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (18,7) ,UO+7200   ,0    ,""    }),  // Jan Sun>=12
                     ({FIXED(355)  ,UO+7200   ,3600 ,""    })});// Dec 20
         case 2021:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (18,7) ,UO+7200   ,0    ,""    })});// Jan Sun>=12
         default: // ..1997 and 2022..
         case 2001..2008:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Guam
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1959:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(178)  ,UO+7200   ,3600 ,"D"   })});// Jun 27
         case 1961:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(29)   ,UO+3600   ,0    ,"S"   })});// Jan 29
         case 1967:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(244)  ,UO+7200   ,3600 ,"D"   })});// Sep 1
         case 1960:
         case 1968:
            return ({({0           ,0         ,3600 ,"D"   })});
         case 1969:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(26)   ,UO-3540   ,0    ,"S"   }),  // Jan 26
                     ({FIXED(173)  ,UO+7200   ,3600 ,"D"   }),  // Jun 22
                     ({FIXED(243)  ,UO+3600   ,0    ,"S"   })});// Aug 31
         case 1970..1971:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (250,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=1
         case 1973:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(350)  ,UO+7200   ,3600 ,"D"   })});// Dec 16
         case 1974:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(55)   ,UO+3600   ,0    ,"S"   })});// Feb 24
         case 1976:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(147)  ,UO+7200   ,3600 ,"D"   }),  // May 26
                     ({FIXED(235)  ,UO+3660   ,0    ,"S"   })});// Aug 22
         case 1977:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(114)  ,UO+7200   ,3600 ,"D"   }),  // Apr 24
                     ({FIXED(240)  ,UO+3600   ,0    ,"S"   })});// Aug 28
         default: // ..1958 and 1978..
         case 1962..1966:
         case 1972:
         case 1975:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class NC
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (341,7),UO+0      ,3600 ,""    })});// Dec Sun>=1
         case 1978:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(58)   ,UO-3600   ,0    ,""    }),  // Feb 27
                     ({LDAY (341,7),UO+0      ,3600 ,""    })});// Dec Sun>=1
         case 1979:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(58)   ,UO-3600   ,0    ,""    })});// Feb 27
         case 1996:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(336)  ,UO+7200   ,3600 ,""    })});// Dec 1
         case 1997:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(61)   ,UO+7200   ,0    ,""    })});// Mar 2
         default: // ..1976 and 1998..
         case 1980..1995:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class NZ
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1926:
            return ({({0           ,0         ,0    ,"M"   })});// ?
         case 1927:
            return ({({0           ,0         ,0    ,"M"   }),  // ?
                     ({FIXED(310)  ,UO+7200   ,3600 ,"S"   })});// Nov 6
         case 1928:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(64)   ,UO+3600   ,0    ,"M"   }),  // Mar 4
                     ({LDAY (288,7),UO+7200   ,1800 ,"S"   })});// Oct Sun>=8
         case 1929..1933:
            return ({({0           ,0         ,1800 ,"S"   }),  
                     ({LDAYL(80,7) ,UO+5400   ,0    ,"M"   }),  // Mar Sun>=15
                     ({LDAYL(287,7),UO+7200   ,1800 ,"S"   })});// Oct Sun>=8
         case 1934..1940:
            return ({({0           ,0         ,1800 ,"S"   }),  
                     ({LDAYL(120,7),UO+5400   ,0    ,"M"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+7200   ,1800 ,"S"   })});// Sep lastSun
         case 1941..1945:
            return ({({0           ,0         ,1800 ,"S"   })});
         case 1946:
            return ({({0           ,0         ,1800 ,"S"   }),  
                     ({FIXED(1)    ,UO-1800   ,0    ,"S"   })});// Jan 1
         case 1947..1973:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1974:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (311,7),UO+7200   ,3600 ,"D"   })});// Nov Sun>=1
         case 1975:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (59,7) ,UO+7200   ,0    ,"S"   }),  // Feb lastSun
                     ({LDAY (304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1976..1988:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+7200   ,3600 ,"D"   })});// Oct lastSun
         case 1989:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (66,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=1
                     ({LDAY (287,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=8
         case 1990..2006:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAYL(280,7),UO+7200   ,3600 ,"D"   })});// Oct Sun>=1
         case 2007:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (80,7) ,UO+7200   ,0    ,"S"   }),  // Mar Sun>=15
                     ({LDAY (273,7),UO+7200   ,3600 ,"D"   })});// Sep lastSun
         case 2008..:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,0    ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO+7200   ,3600 ,"D"   })});// Sep lastSun
      }
   }
}

class Chatham
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1973:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (311,7),UO+9900   ,3600 ,""    })});// Nov Sun>=1
         case 1975:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (59,7) ,UO+9900   ,0    ,""    }),  // Feb lastSun
                     ({LDAY (304,7),UO+9900   ,3600 ,""    })});// Oct lastSun
         case 1976..1988:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(66,7) ,UO+9900   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+9900   ,3600 ,""    })});// Oct lastSun
         case 1989:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO+9900   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAY (287,7),UO+9900   ,3600 ,""    })});// Oct Sun>=8
         case 1990..2006:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(80,7) ,UO+9900   ,0    ,""    }),  // Mar Sun>=15
                     ({LDAYL(280,7),UO+9900   ,3600 ,""    })});// Oct Sun>=1
         case 2007:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (80,7) ,UO+9900   ,0    ,""    }),  // Mar Sun>=15
                     ({LDAY (273,7),UO+9900   ,3600 ,""    })});// Sep lastSun
         case 2008..:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(97,7) ,UO+9900   ,0    ,""    }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO+9900   ,3600 ,""    })});// Sep lastSun
      }
   }
}

class Cook
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(316)  ,UO+0      ,1800 ,""    })});// Nov 12
         case 1979..1990:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAYL(66,7) ,UO-1800   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAYL(304,7),UO+0      ,1800 ,""    })});// Oct lastSun
         case 1991:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (66,7) ,UO-1800   ,0    ,""    })});// Mar Sun>=1
         default: // ..1977 and 1992..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class WS
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 2010:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (273,7),UO+0      ,3600 ,""    })});// Sep lastSun
         case 2011:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (97,6) ,UO+10800  ,0    ,""    }),  // Apr Sat>=1
                     ({LDAY (273,6),UO+10800  ,3600 ,""    })});// Sep lastSat
         case 2012..2020:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(97,7) ,UO+10800  ,0    ,""    }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO+10800  ,3600 ,""    })});// Sep lastSun
         case 2021:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (97,7) ,UO+10800  ,0    ,""    })});// Apr Sun>=1
         default: // ..2009 and 2022..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Tonga
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1999:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(280)  ,UO+7200   ,3600 ,""    })});// Oct 7
         case 2000:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(79)   ,UO+7200   ,0    ,""    }),  // Mar 19
                     ({LDAY (312,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 2001:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (31,7) ,UO+3600   ,0    ,""    }),  // Jan lastSun
                     ({LDAY (311,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 2002:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (31,7) ,UO+3600   ,0    ,""    })});// Jan lastSun
         case 2016:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (312,7),UO+7200   ,3600 ,""    })});// Nov Sun>=1
         case 2017:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (21,7) ,UO+7200   ,0    ,""    })});// Jan Sun>=15
         default: // ..1998 and 2018..
         case 2003..2015:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Vanuatu
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1973:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(356)  ,43200     ,3600 ,""    })});// Dec 22
         case 1974:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(89)   ,43200     ,0    ,""    })});// Mar 30
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (271,6),UO+86400  ,3600 ,""    })});// Sep Sat>=22
         case 1984..1991:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(87,6) ,UO+82800  ,0    ,""    }),  // Mar Sat>=22
                     ({LDAYL(271,6),UO+86400  ,3600 ,""    })});// Sep Sat>=22
         case 1992:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (28,6) ,UO+82800  ,0    ,""    }),  // Jan Sat>=22
                     ({LDAY (302,6),UO+86400  ,3600 ,""    })});// Oct Sat>=22
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (28,6) ,UO+82800  ,0    ,""    })});// Jan Sat>=22
         default: // ..1972 and 1994..
         case 1975..1982:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Ghana
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1919:
            return ({({0           ,0         ,0    ,"GMT" }),  // ?
                     ({FIXED(328)  ,UO+0      ,1200 ,"+0020"})});// Nov 24
         case 1920..1939:
            return ({({0           ,0         ,1200 ,"+0020"}),  
                     ({FIXED(1)    ,UO+6000   ,0    ,"GMT" }),  // Jan 1
                     ({FIX_L(244)  ,UO+7200   ,1200 ,"+0020"})});// Sep 1
         case 1940..1941:
            return ({({0           ,0         ,1200 ,"+0020"}),  
                     ({FIXED(1)    ,UO+6000   ,0    ,"GMT" }),  // Jan 1
                     ({FIX_L(121)  ,UO+7200   ,1200 ,"+0020"})});// May 1
         case 1942:
            return ({({0           ,0         ,1200 ,"+0020"}),  
                     ({FIXED(1)    ,UO+6000   ,0    ,"GMT" })});// Jan 1
         case 1950:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(244)  ,UO+7200   ,1800 ,"+0030"})});// Sep 1
         case 1951..1955:
            return ({({0           ,0         ,1800 ,"+0030"}),  
                     ({FIXED(1)    ,UO+5400   ,0    ,"GMT" }),  // Jan 1
                     ({FIX_L(244)  ,UO+7200   ,1800 ,"+0030"})});// Sep 1
         case 1956:
            return ({({0           ,0         ,1800 ,"+0030"}),  
                     ({FIXED(1)    ,UO+5400   ,0    ,"GMT" })});// Jan 1
         default: // ..1918 and 1957..
         case 1943..1949:
            return ({({0           ,0         ,0    ,"GMT" })});
      }
   }
}

class SL
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1931:
            return ({({0           ,0         ,0    ,"-01" })});// ?
         case 1932:
            return ({({0           ,0         ,0    ,"-01" }),  // ?
                     ({FIXED(336)  ,UO+0      ,1200 ,"-0040"})});// Dec 1
         case 1933..1938:
            return ({({0           ,0         ,1200 ,"-0040"}),  
                     ({FIX_L(90)   ,UO+85200  ,0    ,"-01" }),  // Mar 31
                     ({FIX_L(243)  ,UO+86400  ,1200 ,"-0040"})});// Aug 31
         case 1939:
            return ({({0           ,0         ,1200 ,"-0040"}),  
                     ({FIXED(151)  ,UO+85200  ,0    ,"-01" }),  // May 31
                     ({FIXED(243)  ,UO+86400  ,1200 ,"-0040"})});// Aug 31
         case 1940..:
            return ({({0           ,0         ,1200 ,"-0040"})});
      }
   }
}

class Mont
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1917:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(84)   ,UO+7200   ,3600 ,"D"   }),  // Mar 25
                     ({FIXED(114)  ,UO-3600   ,0    ,"S"   })});// Apr 24
         case 1919:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(90)   ,UO+9000   ,3600 ,"D"   }),  // Mar 31
                     ({FIXED(298)  ,UO+5400   ,0    ,"S"   })});// Oct 25
         case 1920:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(123)  ,UO+9000   ,3600 ,"D"   }),  // May 2
                     ({LDAY (281,7),UO+5400   ,0    ,"S"   })});// Oct Sun>=1
         case 1921:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(121)  ,UO+7200   ,3600 ,"D"   }),  // May 1
                     ({LDAY (280,7),UO+5400   ,0    ,"S"   })});// Oct Sun>=1
         case 1922:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,UO+7200   ,3600 ,"D"   }),  // Apr 30
                     ({LDAY (280,7),UO+5400   ,0    ,"S"   })});// Oct Sun>=1
         case 1924:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(138)  ,UO+7200   ,3600 ,"D"   }),  // May 17
                     ({LDAY (274,7),UO+5400   ,0    ,"S"   })});// Sep lastSun
         case 1925..1926:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (127,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAY (273,7),UO+5400   ,0    ,"S"   })});// Sep lastSun
         case 1927..1937:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,6),UO+86400  ,3600 ,"D"   }),  // Apr lastSat
                     ({LDAYL(273,6),UO+82800  ,0    ,"S"   })});// Sep lastSat
         case 1938..1939:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+0      ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (273,7),UO-3600   ,0    ,"S"   })});// Sep lastSun
         case 1940:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (121,7),UO+0      ,3600 ,"D"   })});// Apr lastSun
         case 1941..1944:
            return ({({0           ,0         ,3600 ,"D"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1946..1948:
         case 1951..1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1949..1950:
         case 1957..1973:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1916 and 1974..
         case 1918:
         case 1923:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Bahamas
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(121)  ,UO+86400  ,3600 ,"W"   })});// May 1
         case 1943:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1944:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(366)  ,UO+82800  ,0    ,"S"   })});// Dec 31
         case 1945:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(32)   ,UO+0      ,3600 ,"W"   }),  // Feb 1
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({FIXED(290)  ,UO+82800  ,0    ,"S"   })});// Oct 17
         case 1964..1975:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1941 and 1976..
         case 1946..1963:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Iceland
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1917:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(50)   ,UO+82800  ,3600 ,""    }),  // Feb 19
                     ({FIXED(294)  ,UO+0      ,0    ,""    })});// Oct 21
         case 1918..1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(50)   ,UO+82800  ,3600 ,""    }),  // Feb 19
                     ({FIXED(320)  ,UO+0      ,0    ,""    })});// Nov 16
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(78)   ,UO+82800  ,3600 ,""    }),  // Mar 19
                     ({FIXED(174)  ,UO+0      ,0    ,""    })});// Jun 23
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(119)  ,UO+82800  ,3600 ,""    }),  // Apr 29
                     ({FIXED(302)  ,UO+3600   ,0    ,""    })});// Oct 29
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(56)   ,UO+7200   ,3600 ,""    }),  // Feb 25
                     ({LDAY (313,7),UO+3600   ,0    ,""    })});// Nov Sun>=2
         case 1941:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (67,7) ,UO+3600   ,3600 ,""    }),  // Mar Sun>=2
                     ({LDAY (312,7),UO+3600   ,0    ,""    })});// Nov Sun>=2
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (67,7) ,UO+3600   ,3600 ,""    }),  // Mar Sun>=2
                     ({LDAY (301,7),UO+3600   ,0    ,""    })});// Oct Sun>=22
         case 1943..1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(66,7) ,UO+3600   ,3600 ,""    }),  // Mar Sun>=1
                     ({LDAYL(301,7),UO+3600   ,0    ,""    })});// Oct Sun>=22
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+3600   ,3600 ,""    }),  // Apr Sun>=1
                     ({FIXED(303)  ,UO+3600   ,0    ,""    })});// Oct 30
         case 1947..1948:
         case 1950..1966:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+3600   ,3600 ,""    }),  // Apr Sun>=1
                     ({LDAYL(301,7),UO+3600   ,0    ,""    })});// Oct Sun>=22
         case 1967:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+3600   ,3600 ,""    }),  // Apr Sun>=1
                     ({FIXED(302)  ,UO+3600   ,0    ,""    })});// Oct 29
         default: // ..1916 and 1968..
         case 1920:
         case 1922..1938:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Neth
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1915:
            return ({({0           ,0         ,0    ,"AMT" })});// ?
         case 1916:
            return ({({0           ,0         ,0    ,"AMT" }),  // ?
                     ({FIXED(122)  ,UO+0      ,3600 ,"NST" }),  // May 1
                     ({FIXED(275)  ,UO-3600   ,0    ,"AMT" })});// Oct 1
         case 1917:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({FIXED(106)  ,UO+7200   ,3600 ,"NST" }),  // Apr 16
                     ({FIXED(260)  ,UO+7200   ,0    ,"AMT" })});// Sep 17
         case 1918..1921:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({LDAYL(97,1) ,UO+7200   ,3600 ,"NST" }),  // Apr Mon>=1
                     ({LDAYL(273,1),UO+7200   ,0    ,"AMT" })});// Sep lastMon
         case 1922:
         case 1924:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"NST" }),  // Mar lastSun
                     ({LDAYL(281,7),UO+7200   ,0    ,"AMT" })});// Oct Sun>=2
         case 1923:
         case 1925:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({LDAY (158,5),UO+7200   ,3600 ,"NST" }),  // Jun Fri>=1
                     ({LDAY (281,7),UO+7200   ,0    ,"AMT" })});// Oct Sun>=2
         case 1932:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({FIXED(143)  ,UO+7200   ,3600 ,"NST" }),  // May 22
                     ({LDAY (282,7),UO+7200   ,0    ,"AMT" })});// Oct Sun>=2
         case 1926..1931:
         case 1933..1936:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({FIX_L(135)  ,UO+7200   ,3600 ,"NST" }),  // May 15
                     ({LDAYL(281,7),UO+7200   ,0    ,"AMT" })});// Oct Sun>=2
         case 1937:
            return ({({0           ,0         ,0    ,"AMT" }),  
                     ({FIXED(142)  ,UO+7200   ,3600 ,"NST" }),  // May 22
                     ({FIXED(182)  ,UO-3600   ,3600 ,"S"   }),  // Jul 1
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1938..1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(135)  ,UO+7200   ,3600 ,"S"   }),  // May 15
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(259)  ,UO+7200   ,0    ,""    })});// Sep 16
         case 1940..1944:
         case 1946..:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Denmark
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(135)  ,UO+82800  ,3600 ,"S"   }),  // May 14
                     ({FIXED(274)  ,UO+79200  ,0    ,""    })});// Sep 30
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(136)  ,UO+0      ,3600 ,"S"   })});// May 15
         case 1941..1944:
            return ({({0           ,0         ,3600 ,"S"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(227)  ,UO+7200   ,0    ,""    })});// Aug 15
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+7200   ,3600 ,"S"   }),  // May 1
                     ({FIXED(244)  ,UO+7200   ,0    ,""    })});// Sep 1
         case 1947:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(124)  ,UO+7200   ,3600 ,"S"   }),  // May 4
                     ({FIXED(222)  ,UO+7200   ,0    ,""    })});// Aug 10
         case 1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(130)  ,UO+7200   ,3600 ,"S"   }),  // May 9
                     ({FIXED(221)  ,UO+7200   ,0    ,""    })});// Aug 8
         default: // ..1915 and 1949..
         case 1917..1939:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Lux
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1915:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(135)  ,UO+82800  ,3600 ,"S"   }),  // May 14
                     ({FIXED(275)  ,UO+0      ,0    ,""    })});// Oct 1
         case 1917:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(118)  ,UO+82800  ,3600 ,"S"   }),  // Apr 28
                     ({FIXED(260)  ,UO+0      ,0    ,""    })});// Sep 17
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (111,1),UO+7200   ,3600 ,"S"   }),  // Apr Mon>=15
                     ({LDAY (264,1),UO+7200   ,0    ,""    })});// Sep Mon>=15
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+82800  ,3600 ,"S"   }),  // Mar 1
                     ({FIXED(278)  ,UO+7200   ,0    ,""    })});// Oct 5
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(45)   ,UO+82800  ,3600 ,"S"   }),  // Feb 14
                     ({FIXED(298)  ,UO+3600   ,0    ,""    })});// Oct 24
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(73)   ,UO+82800  ,3600 ,"S"   }),  // Mar 14
                     ({FIXED(299)  ,UO+3600   ,0    ,""    })});// Oct 26
         case 1922:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(84)   ,UO+82800  ,3600 ,"S"   }),  // Mar 25
                     ({LDAY (281,7),UO+0      ,0    ,""    })});// Oct Sun>=2
         case 1923:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(111)  ,UO+82800  ,3600 ,"S"   }),  // Apr 21
                     ({LDAY (281,7),UO+3600   ,0    ,""    })});// Oct Sun>=2
         case 1924:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+82800  ,3600 ,"S"   }),  // Mar 29
                     ({LDAY (282,7),UO+0      ,0    ,""    })});// Oct Sun>=2
         case 1925:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(95)   ,UO+82800  ,3600 ,"S"   }),  // Apr 5
                     ({LDAY (281,7),UO+0      ,0    ,""    })});// Oct Sun>=2
         case 1926:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 17
                     ({LDAY (281,7),UO+0      ,0    ,""    })});// Oct Sun>=2
         case 1927:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+82800  ,3600 ,"S"   }),  // Apr 9
                     ({LDAY (281,7),UO+0      ,0    ,""    })});// Oct Sun>=2
         case 1928:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 14
                     ({LDAY (282,7),UO+0      ,0    ,""    })});// Oct Sun>=2
         case 1929:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(110)  ,UO+82800  ,3600 ,"S"   })});// Apr 20
         case 1930..:
            return ({({0           ,0         ,3600 ,"S"   })});
      }
   }
}

class Norway
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(143)  ,UO+3600   ,3600 ,"S"   }),  // May 22
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Sep 30
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1959..1964:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(80,7) ,UO+7200   ,3600 ,"S"   }),  // Mar Sun>=15
                     ({LDAYL(264,7),UO+7200   ,0    ,""    })});// Sep Sun>=15
         case 1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(115)  ,UO+7200   ,3600 ,"S"   }),  // Apr 25
                     ({LDAY (264,7),UO+7200   ,0    ,""    })});// Sep Sun>=15
         default: // ..1915 and 1966..
         case 1917..1944:
         case 1946..1958:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class GB_Eire
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,"GMT" }),  // ?
                     ({FIXED(142)  ,UO+7200   ,3600 ,"BST" }),  // May 21
                     ({FIXED(275)  ,UO+7200   ,0    ,"GMT" })});// Oct 1
         case 1917:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(98)   ,UO+7200   ,3600 ,"BST" }),  // Apr 8
                     ({FIXED(260)  ,UO+7200   ,0    ,"GMT" })});// Sep 17
         case 1918:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(83)   ,UO+7200   ,3600 ,"BST" }),  // Mar 24
                     ({FIXED(273)  ,UO+7200   ,0    ,"GMT" })});// Sep 30
         case 1919:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(89)   ,UO+7200   ,3600 ,"BST" }),  // Mar 30
                     ({FIXED(272)  ,UO+7200   ,0    ,"GMT" })});// Sep 29
         case 1920:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(88)   ,UO+7200   ,3600 ,"BST" }),  // Mar 28
                     ({FIXED(299)  ,UO+7200   ,0    ,"GMT" })});// Oct 25
         case 1921:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(93)   ,UO+7200   ,3600 ,"BST" }),  // Apr 3
                     ({FIXED(276)  ,UO+7200   ,0    ,"GMT" })});// Oct 3
         case 1922:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(85)   ,UO+7200   ,3600 ,"BST" }),  // Mar 26
                     ({FIXED(281)  ,UO+7200   ,0    ,"GMT" })});// Oct 8
         case 1923:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAY (112,7),UO+7200   ,3600 ,"BST" }),  // Apr Sun>=16
                     ({LDAY (265,7),UO+7200   ,0    ,"GMT" })});// Sep Sun>=16
         case 1924:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAY (106,7),UO+7200   ,3600 ,"BST" }),  // Apr Sun>=9
                     ({LDAY (266,7),UO+7200   ,0    ,"GMT" })});// Sep Sun>=16
         case 1939:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAY (112,7),UO+7200   ,3600 ,"BST" }),  // Apr Sun>=16
                     ({LDAY (326,7),UO+7200   ,0    ,"GMT" })});// Nov Sun>=16
         case 1940:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAY (61,7) ,UO+7200   ,3600 ,"BST" })});// Feb Sun>=23
         case 1941:
            return ({({0           ,0         ,3600 ,"BST" }),  
                     ({LDAY (128,7),UO+3600   ,7200 ,"BDST"}),  // May Sun>=2
                     ({LDAY (227,7),UO+3600   ,3600 ,"BST" })});// Aug Sun>=9
         case 1942..1943:
            return ({({0           ,0         ,3600 ,"BST" }),  
                     ({LDAY (98,7) ,UO+3600   ,7200 ,"BDST"}),  // Apr Sun>=2
                     ({LDAY (227,7),UO+3600   ,3600 ,"BST" })});// Aug Sun>=9
         case 1944:
            return ({({0           ,0         ,3600 ,"BST" }),  
                     ({LDAY (99,7) ,UO+3600   ,7200 ,"BDST"}),  // Apr Sun>=2
                     ({LDAY (266,7),UO+3600   ,3600 ,"BST" })});// Sep Sun>=16
         case 1945:
            return ({({0           ,0         ,3600 ,"BST" }),  
                     ({LDAY (98,1) ,UO+3600   ,7200 ,"BDST"}),  // Apr Mon>=2
                     ({LDAY (196,7),UO+3600   ,3600 ,"BST" }),  // Jul Sun>=9
                     ({LDAY (281,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=2
         case 1947:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(75)   ,UO+7200   ,3600 ,"BST" }),  // Mar 16
                     ({FIXED(103)  ,UO+3600   ,7200 ,"BDST"}),  // Apr 13
                     ({FIXED(222)  ,UO+3600   ,3600 ,"BST" }),  // Aug 10
                     ({FIXED(306)  ,UO+7200   ,0    ,"GMT" })});// Nov 2
         case 1948:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(74)   ,UO+7200   ,3600 ,"BST" }),  // Mar 14
                     ({FIXED(305)  ,UO+7200   ,0    ,"GMT" })});// Oct 31
         case 1949:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(93)   ,UO+7200   ,3600 ,"BST" }),  // Apr 3
                     ({FIXED(303)  ,UO+7200   ,0    ,"GMT" })});// Oct 30
         case 1950..1952:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(110,7),UO+7200   ,3600 ,"BST" }),  // Apr Sun>=14
                     ({LDAYL(300,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=21
         case 1925..1926:
         case 1928..1929:
         case 1931..1932:
         case 1934:
         case 1936..1937:
         case 1953:
         case 1955..1956:
         case 1958..1959:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(112,7),UO+7200   ,3600 ,"BST" }),  // Apr Sun>=16
                     ({LDAYL(281,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=2
         case 1927:
         case 1930:
         case 1933:
         case 1935:
         case 1938:
         case 1946:
         case 1954:
         case 1957:
         case 1960:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(105,7),UO+7200   ,3600 ,"BST" }),  // Apr Sun>=9
                     ({LDAYL(281,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=2
         case 1961..1963:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,"BST" }),  // Mar lastSun
                     ({LDAY (302,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=23
         case 1964..1967:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(84,7) ,UO+7200   ,3600 ,"BST" }),  // Mar Sun>=19
                     ({LDAYL(302,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=23
         case 1968:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({FIXED(49)   ,UO+7200   ,3600 ,"BST" }),  // Feb 18
                     ({LDAY (303,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=23
         case 1972..1980:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(81,7) ,UO+7200   ,3600 ,"BST" }),  // Mar Sun>=16
                     ({LDAYL(302,7),UO+7200   ,0    ,"GMT" })});// Oct Sun>=23
         case 1981..1989:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(90,7) ,3600      ,3600 ,"BST" }),  // Mar lastSun
                     ({LDAYL(302,7),3600      ,0    ,"GMT" })});// Oct Sun>=23
         case 1990..1995:
            return ({({0           ,0         ,0    ,"GMT" }),  
                     ({LDAYL(90,7) ,3600      ,3600 ,"BST" }),  // Mar lastSun
                     ({LDAYL(301,7),3600      ,0    ,"GMT" })});// Oct Sun>=22
         default: // ..1915 and 1996..
         case 1969..1971:
            return ({({0           ,0         ,0    ,"GMT" })});
      }
   }
}

class Eire
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1970:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1971:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(304)  ,7200      ,-3600,""    })});// Oct 31
         case 1972..1980:
            return ({({0           ,0         ,-3600,""    }),  
                     ({LDAYL(81,7) ,7200      ,0    ,""    }),  // Mar Sun>=16
                     ({LDAYL(302,7),7200      ,-3600,""    })});// Oct Sun>=23
         case 1981..1989:
            return ({({0           ,0         ,-3600,""    }),  
                     ({LDAYL(90,7) ,3600      ,0    ,""    }),  // Mar lastSun
                     ({LDAYL(302,7),3600      ,-3600,""    })});// Oct Sun>=23
         case 1990..1995:
            return ({({0           ,0         ,-3600,""    }),  
                     ({LDAYL(90,7) ,3600      ,0    ,""    }),  // Mar lastSun
                     ({LDAYL(301,7),3600      ,-3600,""    })});// Oct Sun>=22
         case 1996..:
            return ({({0           ,0         ,-3600,""    }),  
                     ({LDAYL(90,7) ,3600      ,0    ,""    }),  // Mar lastSun
                     ({LDAYL(304,7),3600      ,-3600,""    })});// Oct lastSun
      }
   }
}

class EU
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1976:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,3600      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(274)  ,3600      ,0    ,""    })});// Oct 1
         case 1977:
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,3600      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(273,7),3600      ,0    ,""    })});// Sep lastSun
         case 1981..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,3600      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),3600      ,0    ,""    })});// Sep lastSun
         case 1996..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,3600      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),3600      ,0    ,""    })});// Oct lastSun
      }
   }
}

class W_Eur
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1976:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+3600   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(274)  ,UO+3600   ,0    ,""    })});// Oct 1
         case 1977:
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+3600   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1981..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+3600   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1996..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+3600   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,""    })});// Oct lastSun
      }
   }
}

class C_Eur
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(121)  ,UO+82800  ,3600 ,"S"   }),  // Apr 30
                     ({FIXED(275)  ,UO+0      ,0    ,""    })});// Oct 1
         case 1917..1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (111,1),UO+7200   ,3600 ,"S"   }),  // Apr Mon>=15
                     ({LDAY (264,1),UO+7200   ,0    ,""    })});// Sep Mon>=15
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   })});// Apr 1
         case 1941:
            return ({({0           ,0         ,3600 ,"S"   })});
         case 1942:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(306)  ,UO+7200   ,0    ,""    })});// Nov 2
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+7200   ,3600 ,"S"   }),  // Mar 29
                     ({FIXED(277)  ,UO+7200   ,0    ,""    })});// Oct 4
         case 1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (98,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(276)  ,UO+7200   ,0    ,""    })});// Oct 2
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(259)  ,UO+7200   ,0    ,""    })});// Sep 16
         default: // ..1915:
         case 1919..1939:
         case 1946..1976:
            return ({({0           ,0         ,0    ,""    })});
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1977:
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1981..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1996..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,0    ,""    })});// Oct lastSun
      }
   }
}

class E_Eur
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1976:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1977:
         case 1979..1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1981..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1996..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO-3600   ,0    ,""    })});// Oct lastSun
      }
   }
}

class Russia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1916:
            return ({({0           ,0         ,0    ,"MMT" })});// ?
         case 1917:
            return ({({0           ,0         ,0    ,"MMT" }),  // ?
                     ({FIXED(182)  ,UO+82800  ,3600 ,"MST" }),  // Jul 1
                     ({FIXED(362)  ,UO-3600   ,0    ,"MMT" })});// Dec 28
         case 1918:
            return ({({0           ,0         ,0    ,"MMT" }),  
                     ({FIXED(151)  ,UO+79200  ,7200 ,"MDST"}),  // May 31
                     ({FIXED(259)  ,UO-3600   ,3600 ,"MST" })});// Sep 16
         case 1919:
            return ({({0           ,0         ,3600 ,"MST" }),  
                     ({FIXED(151)  ,UO+79200  ,7200 ,"MDST"}),  // May 31
                     ({FIXED(182)  ,0         ,3600 ,"MSD" }),  // Jul 1
                     ({FIXED(228)  ,UO-3600   ,0    ,"MSK" })});// Aug 16
         case 1920:
            return ({({0           ,0         ,0    ,"MSK" })});
         case 1921:
            return ({({0           ,0         ,0    ,"MSK" }),  
                     ({FIXED(45)   ,UO+82800  ,3600 ,"MSD" }),  // Feb 14
                     ({FIXED(79)   ,UO+79200  ,7200 ,"+05" }),  // Mar 20
                     ({FIXED(244)  ,UO-7200   ,3600 ,"MSD" }),  // Sep 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1981..1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1984:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({LDAY (274,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1985..1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1996..2010:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,0    ,""    })});// Oct lastSun
         case 1922..1980:
         case 2011..:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Albania
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(168)  ,UO+0      ,3600 ,"S"   })});// Jun 16
         case 1942:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(306)  ,UO+7200   ,0    ,""    })});// Nov 2
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+7200   ,3600 ,"S"   }),  // Mar 29
                     ({FIXED(100)  ,UO+7200   ,0    ,""    })});// Apr 10
         default: // ..1939:
         case 1944..1973:
            return ({({0           ,0         ,0    ,""    })});
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(124)  ,UO+0      ,3600 ,"S"   }),  // May 4
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 2
         case 1975:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 2
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(128)  ,UO+0      ,3600 ,"S"   }),  // May 8
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 2
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(126)  ,UO+0      ,3600 ,"S"   }),  // May 6
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(125)  ,UO+0      ,3600 ,"S"   }),  // May 5
                     ({FIXED(273)  ,UO-3600   ,0    ,""    })});// Sep 30
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(124)  ,UO+0      ,3600 ,"S"   }),  // May 3
                     ({FIXED(278)  ,UO-3600   ,0    ,""    })});// Oct 4
         case 1981:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(116)  ,UO+0      ,3600 ,"S"   }),  // Apr 26
                     ({FIXED(270)  ,UO-3600   ,0    ,""    })});// Sep 27
         case 1976:
         case 1982:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(122)  ,UO+0      ,3600 ,"S"   }),  // May 2
                     ({FIX_L(276)  ,UO-3600   ,0    ,""    })});// Oct 3
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(108)  ,UO+0      ,3600 ,"S"   }),  // Apr 18
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1984:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,"S"   })});// Apr 1
         case 1941:
         case 1985..:
            return ({({0           ,0         ,3600 ,"S"   })});
      }
   }
}

class Austria
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(96)   ,UO+7200   ,3600 ,"S"   }),  // Apr 5
                     ({FIXED(257)  ,UO+7200   ,0    ,""    })});// Sep 13
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(104)  ,UO+7200   ,3600 ,"S"   }),  // Apr 14
                     ({FIXED(280)  ,UO+7200   ,0    ,""    })});// Oct 7
         case 1947:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+7200   ,3600 ,"S"   }),  // Apr 6
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(109)  ,UO+7200   ,3600 ,"S"   }),  // Apr 18
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(97)   ,UO+0      ,3600 ,"S"   }),  // Apr 6
                     ({FIXED(272)  ,UO-3600   ,0    ,""    })});// Sep 28
         default: // ..1919 and 1981..
         case 1921..1945:
         case 1949..1979:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Belgium
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(68)   ,UO+0      ,3600 ,"S"   }),  // Mar 9
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+82800  ,3600 ,"S"   }),  // Mar 1
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(45)   ,UO+82800  ,3600 ,"S"   }),  // Feb 14
                     ({FIXED(297)  ,UO+82800  ,0    ,""    })});// Oct 23
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(73)   ,UO+82800  ,3600 ,"S"   }),  // Mar 14
                     ({FIXED(298)  ,UO+82800  ,0    ,""    })});// Oct 25
         case 1922:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(84)   ,UO+82800  ,3600 ,"S"   }),  // Mar 25
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1923:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(111)  ,UO+82800  ,3600 ,"S"   }),  // Apr 21
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1924:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+82800  ,3600 ,"S"   }),  // Mar 29
                     ({LDAY (281,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1925:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+82800  ,3600 ,"S"   }),  // Apr 4
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1926:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 17
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1927:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+82800  ,3600 ,"S"   }),  // Apr 9
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1928:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 14
                     ({LDAY (282,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1929:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(111)  ,UO+7200   ,3600 ,"S"   }),  // Apr 21
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1930:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(103)  ,UO+7200   ,3600 ,"S"   }),  // Apr 13
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1932:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+7200   ,3600 ,"S"   }),  // Apr 3
                     ({LDAY (282,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1933:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+7200   ,3600 ,"S"   }),  // Mar 26
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1934:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(98)   ,UO+7200   ,3600 ,"S"   }),  // Apr 8
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1935:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(90)   ,UO+7200   ,3600 ,"S"   }),  // Mar 31
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1931:
         case 1936:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(109)  ,UO+7200   ,3600 ,"S"   }),  // Apr 19
                     ({LDAYL(281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1937:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+7200   ,3600 ,"S"   }),  // Apr 4
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1938:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(86)   ,UO+7200   ,3600 ,"S"   }),  // Mar 27
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=2
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(106)  ,UO+7200   ,3600 ,"S"   }),  // Apr 16
                     ({FIXED(323)  ,UO+7200   ,0    ,""    })});// Nov 19
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(56)   ,UO+7200   ,3600 ,"S"   })});// Feb 25
         case 1941..1943:
            return ({({0           ,0         ,3600 ,"S"   })});
         case 1944:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(261)  ,UO+7200   ,0    ,""    })});// Sep 17
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(259)  ,UO+7200   ,0    ,""    })});// Sep 16
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(139)  ,UO+7200   ,3600 ,"S"   }),  // May 19
                     ({FIXED(280)  ,UO+7200   ,0    ,""    })});// Oct 7
         default: // ..1917 and 1947..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Bulg
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1978:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(90)   ,UO+82800  ,3600 ,"S"   }),  // Mar 31
                     ({FIXED(274)  ,UO+0      ,0    ,""    })});// Oct 1
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (98,6) ,UO+82800  ,3600 ,"S"   }),  // Apr Sat>=1
                     ({FIXED(273)  ,UO+0      ,0    ,""    })});// Sep 29
         case 1981:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,6) ,UO+82800  ,3600 ,"S"   }),  // Apr Sat>=1
                     ({FIXED(270)  ,UO+3600   ,0    ,""    })});// Sep 27
         case 1982:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,6) ,UO+82800  ,3600 ,"S"   })});// Apr Sat>=1
         case 1983..:
            return ({({0           ,0         ,3600 ,"S"   })});
      }
   }
}

class Czech
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (97,1) ,UO+7200   ,3600 ,"S"   }),  // Apr Mon>=1
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(126)  ,UO+7200   ,3600 ,"S"   }),  // May 6
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1947..1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(111,7),UO+7200   ,3600 ,"S"   }),  // Apr Sun>=15
                     ({LDAYL(280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+7200   ,3600 ,"S"   }),  // Apr 9
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         default: // ..1944 and 1950..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Thule
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1990:
            return ({({0           ,0         ,0    ,"S"   })});// ?
         case 1991..1992:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"D"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1993..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 2007..:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+7200   ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO+3600   ,0    ,"S"   })});// Nov Sun>=1
      }
   }
}

class Finland
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(92)   ,UO+86400  ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(277)  ,UO+0      ,0    ,""    })});// Oct 4
         case 1981..1982:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAY (273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         default: // ..1941 and 1983..
         case 1943..1980:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class France
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(166)  ,UO+82800  ,3600 ,"S"   }),  // Jun 14
                     ({LDAY (281,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1917:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(83)   ,UO+82800  ,3600 ,"S"   }),  // Mar 24
                     ({LDAY (280,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(68)   ,UO+82800  ,3600 ,"S"   }),  // Mar 9
                     ({LDAY (280,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+82800  ,3600 ,"S"   }),  // Mar 1
                     ({LDAY (280,7),UO+82800  ,0    ,""    })});// Oct Sun>=1
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(45)   ,UO+82800  ,3600 ,"S"   }),  // Feb 14
                     ({FIXED(297)  ,UO+82800  ,0    ,""    })});// Oct 23
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(73)   ,UO+82800  ,3600 ,"S"   }),  // Mar 14
                     ({FIXED(298)  ,UO+82800  ,0    ,""    })});// Oct 25
         case 1923:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(146)  ,UO+82800  ,3600 ,"S"   }),  // May 26
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1924:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+82800  ,3600 ,"S"   }),  // Mar 29
                     ({LDAY (281,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1925:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+82800  ,3600 ,"S"   }),  // Apr 4
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1926:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 17
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1927:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+82800  ,3600 ,"S"   }),  // Apr 9
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1928:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 14
                     ({LDAY (281,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1929:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(110)  ,UO+82800  ,3600 ,"S"   }),  // Apr 20
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1930:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(102)  ,UO+82800  ,3600 ,"S"   }),  // Apr 12
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1932:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+82800  ,3600 ,"S"   }),  // Apr 2
                     ({LDAY (281,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1922:
         case 1933:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(84)   ,UO+82800  ,3600 ,"S"   }),  // Mar 25
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1934:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(97)   ,UO+82800  ,3600 ,"S"   }),  // Apr 7
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1935:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+82800  ,3600 ,"S"   }),  // Mar 30
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1931:
         case 1936:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(108)  ,UO+82800  ,3600 ,"S"   }),  // Apr 18
                     ({LDAYL(280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1937:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+82800  ,3600 ,"S"   }),  // Apr 3
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1938:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+82800  ,3600 ,"S"   }),  // Mar 26
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(322)  ,UO+82800  ,0    ,""    })});// Nov 18
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(56)   ,UO+7200   ,3600 ,"S"   })});// Feb 25
         case 1941:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(125)  ,UO-3600   ,7200 ,"M"   }),  // May 5
                     ({FIXED(279)  ,UO-7200   ,3600 ,"S"   })});// Oct 6
         case 1942:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(68)   ,UO-3600   ,7200 ,"M"   }),  // Mar 9
                     ({FIXED(306)  ,UO+3600   ,3600 ,"S"   })});// Nov 2
         case 1943:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(88)   ,UO+3600   ,7200 ,"M"   }),  // Mar 29
                     ({FIXED(277)  ,UO+3600   ,3600 ,"S"   })});// Oct 4
         case 1944:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(94)   ,UO+3600   ,7200 ,"M"   }),  // Apr 3
                     ({FIXED(282)  ,UO-3600   ,3600 ,"S"   })});// Oct 8
         case 1945:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(92)   ,UO+3600   ,7200 ,"M"   }),  // Apr 2
                     ({FIXED(259)  ,UO+3600   ,0    ,""    })});// Sep 16
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+3600   ,3600 ,"S"   }),  // Mar 28
                     ({FIXED(270)  ,UO+0      ,0    ,""    })});// Sep 26
         default: // ..1915 and 1977..
         case 1946..1975:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Germany
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(104)  ,UO+7200   ,3600 ,"S"   }),  // Apr 14
                     ({FIXED(280)  ,UO+7200   ,0    ,""    })});// Oct 7
         case 1947:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+10800  ,3600 ,"S"   }),  // Apr 6
                     ({FIXED(131)  ,UO+7200   ,7200 ,"M"   }),  // May 11
                     ({FIXED(180)  ,UO+3600   ,3600 ,"S"   }),  // Jun 29
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(109)  ,UO+7200   ,3600 ,"S"   }),  // Apr 18
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(100)  ,UO+7200   ,3600 ,"S"   }),  // Apr 10
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         default: // ..1945 and 1950..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class SovietZone
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(144)  ,UO+7200   ,7200 ,"M"   }),  // May 24
                     ({FIXED(267)  ,UO+3600   ,3600 ,"S"   }),  // Sep 24
                     ({FIXED(322)  ,UO+7200   ,0    ,""    })});// Nov 18
         default: // ..1944 and 1946..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Greece
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1932:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(189)  ,UO+0      ,3600 ,"S"   }),  // Jul 7
                     ({FIXED(245)  ,UO-3600   ,0    ,""    })});// Sep 1
         case 1941:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(97)   ,UO+0      ,3600 ,"S"   })});// Apr 7
         case 1942:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(306)  ,UO+7200   ,0    ,""    })});// Nov 2
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+0      ,3600 ,"S"   }),  // Mar 30
                     ({FIXED(277)  ,UO-3600   ,0    ,""    })});// Oct 4
         case 1952:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(183)  ,UO+0      ,3600 ,"S"   }),  // Jul 1
                     ({FIXED(307)  ,UO-3600   ,0    ,""    })});// Nov 2
         case 1975:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(102)  ,UO+0      ,3600 ,"S"   }),  // Apr 12
                     ({FIXED(330)  ,UO+0      ,0    ,""    })});// Nov 26
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(102)  ,UO+7200   ,3600 ,"S"   }),  // Apr 11
                     ({FIXED(284)  ,UO+7200   ,0    ,""    })});// Oct 10
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(269)  ,UO+7200   ,0    ,""    })});// Sep 26
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(267)  ,UO+10800  ,0    ,""    })});// Sep 24
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+32400  ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(272)  ,UO+3600   ,0    ,""    })});// Sep 29
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,"S"   }),  // Apr 1
                     ({FIXED(272)  ,UO-3600   ,0    ,""    })});// Sep 28
         default: // ..1931 and 1981..
         case 1933..1940:
         case 1944..1951:
         case 1953..1974:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Hungary
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918..1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+7200   ,3600 ,"S"   }),  // Apr 15
                     ({LDAY (264,1),UO+7200   ,0    ,""    })});// Sep Mon>=15
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+7200   ,3600 ,"S"   }),  // Apr 5
                     ({LDAY (265,1),UO+7200   ,0    ,""    })});// Sep Mon>=15
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+82800  ,3600 ,"S"   }),  // May 1
                     ({FIXED(305)  ,UO+0      ,0    ,""    })});// Nov 1
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(90)   ,UO+7200   ,3600 ,"S"   }),  // Mar 31
                     ({FIXED(280)  ,UO+3600   ,0    ,""    })});// Oct 7
         case 1947..1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(100,7),UO+7200   ,3600 ,"S"   }),  // Apr Sun>=4
                     ({LDAYL(280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1954:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(143)  ,UO+0      ,3600 ,"S"   }),  // May 23
                     ({FIXED(276)  ,UO-3600   ,0    ,""    })});// Oct 3
         case 1955:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(142)  ,UO+7200   ,3600 ,"S"   }),  // May 22
                     ({FIXED(275)  ,UO+7200   ,0    ,""    })});// Oct 2
         case 1956..1957:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(158,7),UO+7200   ,3600 ,"S"   }),  // Jun Sun>=1
                     ({LDAYL(273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(97)   ,UO+0      ,3600 ,"S"   }),  // Apr 6
                     ({FIXED(272)  ,UO+0      ,0    ,""    })});// Sep 28
         case 1981..1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         default: // ..1917 and 1984..
         case 1921..1944:
         case 1950..1953:
         case 1958..1979:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Italy
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(155)  ,UO+86400  ,3600 ,"S"   }),  // Jun 3
                     ({FIXED(274)  ,UO+82800  ,0    ,""    })});// Sep 30
         case 1917:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(90)   ,UO+86400  ,3600 ,"S"   }),  // Mar 31
                     ({FIXED(273)  ,UO+82800  ,0    ,""    })});// Sep 30
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(68)   ,UO+86400  ,3600 ,"S"   }),  // Mar 9
                     ({FIXED(279)  ,UO+82800  ,0    ,""    })});// Oct 6
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+86400  ,3600 ,"S"   }),  // Mar 1
                     ({FIXED(277)  ,UO+82800  ,0    ,""    })});// Oct 4
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(80)   ,UO+86400  ,3600 ,"S"   }),  // Mar 20
                     ({FIXED(262)  ,UO+82800  ,0    ,""    })});// Sep 18
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(166)  ,UO+86400  ,3600 ,"S"   })});// Jun 14
         case 1941:
            return ({({0           ,0         ,3600 ,"S"   })});
         case 1942:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(306)  ,UO+7200   ,0    ,""    })});// Nov 2
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+7200   ,3600 ,"S"   }),  // Mar 29
                     ({FIXED(277)  ,UO+7200   ,0    ,""    })});// Oct 4
         case 1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(261)  ,UO+7200   ,0    ,""    })});// Sep 17
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(258)  ,UO+0      ,0    ,""    })});// Sep 15
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(76)   ,UO+7200   ,3600 ,"S"   }),  // Mar 17
                     ({FIXED(279)  ,UO+7200   ,0    ,""    })});// Oct 6
         case 1947:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(75)   ,UO+0      ,3600 ,"S"   }),  // Mar 16
                     ({FIXED(278)  ,UO+0      ,0    ,""    })});// Oct 5
         case 1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+7200   ,3600 ,"S"   }),  // Feb 29
                     ({FIXED(277)  ,UO+7200   ,0    ,""    })});// Oct 3
         case 1966:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (148,7),UO+0      ,3600 ,"S"   }),  // May Sun>=22
                     ({FIXED(267)  ,UO+82800  ,0    ,""    })});// Sep 24
         case 1967..1968:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(148,7),UO+0      ,3600 ,"S"   }),  // May Sun>=22
                     ({LDAYL(271,7),UO+0      ,0    ,""    })});// Sep Sun>=22
         case 1969:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({LDAY (271,7),UO+0      ,0    ,""    })});// Sep Sun>=22
         case 1970:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(151)  ,UO+0      ,3600 ,"S"   }),  // May 31
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1973:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(154)  ,UO+0      ,3600 ,"S"   }),  // Jun 3
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(146)  ,UO+0      ,3600 ,"S"   }),  // May 26
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1975:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(151)  ,UO+0      ,3600 ,"S"   }),  // May 30
                     ({LDAY (274,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1971:
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (148,7),UO+0      ,3600 ,"S"   }),  // May Sun>=22
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1972:
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(148,7),UO+0      ,3600 ,"S"   }),  // May Sun>=22
                     ({FIX_L(274)  ,UO+0      ,0    ,""    })});// Oct 1
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (148,7),UO+0      ,3600 ,"S"   }),  // May Sun>=22
                     ({FIXED(273)  ,UO+0      ,0    ,""    })});// Sep 30
         default: // ..1915 and 1980..
         case 1921..1939:
         case 1949..1965:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Latvia
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1989..1996:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+7200   ,0    ,""    })});// Sep lastSun
         default: // ..1988 and 1997..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Malta
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1973:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(90)   ,UO+0      ,3600 ,"S"   }),  // Mar 31
                     ({FIXED(272)  ,UO+0      ,0    ,""    })});// Sep 29
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(111)  ,UO+0      ,3600 ,"S"   }),  // Apr 21
                     ({FIXED(259)  ,UO+0      ,0    ,""    })});// Sep 16
         case 1975..1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(111,7),UO+7200   ,3600 ,"S"   }),  // Apr Sun>=15
                     ({LDAYL(264,7),UO+3600   ,0    ,""    })});// Sep Sun>=15
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+7200   ,3600 ,"S"   }),  // Mar 31
                     ({LDAY (265,7),UO+3600   ,0    ,""    })});// Sep Sun>=15
         default: // ..1972 and 1981..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Moldova
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1996:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1997..:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+7200   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+7200   ,0    ,""    })});// Oct lastSun
      }
   }
}

class Poland
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(259)  ,UO+7200   ,0    ,""    })});// Sep 16
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+7200   ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(259)  ,UO+7200   ,0    ,""    })});// Sep 16
         case 1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+7200   ,3600 ,"S"   }),  // Apr 3
                     ({FIXED(278)  ,UO+3600   ,0    ,""    })});// Oct 4
         case 1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(119)  ,UO+0      ,3600 ,"S"   }),  // Apr 29
                     ({FIXED(305)  ,UO-3600   ,0    ,""    })});// Nov 1
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(104)  ,UO+0      ,3600 ,"S"   }),  // Apr 14
                     ({FIXED(280)  ,UO+7200   ,0    ,""    })});// Oct 7
         case 1947:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(124)  ,UO+7200   ,3600 ,"S"   }),  // May 4
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(109)  ,UO+7200   ,3600 ,"S"   }),  // Apr 18
                     ({LDAY (281,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(100)  ,UO+7200   ,3600 ,"S"   }),  // Apr 10
                     ({LDAY (280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         case 1957:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(153)  ,UO+3600   ,3600 ,"S"   }),  // Jun 2
                     ({LDAY (273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1958:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+3600   ,3600 ,"S"   }),  // Mar 30
                     ({LDAY (273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1959:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(151)  ,UO+3600   ,3600 ,"S"   }),  // May 31
                     ({LDAY (280,7),UO+3600   ,0    ,""    })});// Oct Sun>=1
         case 1960:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+3600   ,3600 ,"S"   }),  // Apr 3
                     ({LDAY (281,7),UO+3600   ,0    ,""    })});// Oct Sun>=1
         case 1961:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (151,7),UO+3600   ,3600 ,"S"   }),  // May lastSun
                     ({LDAY (280,7),UO+3600   ,0    ,""    })});// Oct Sun>=1
         case 1962..1964:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(151,7),UO+3600   ,3600 ,"S"   }),  // May lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         default: // ..1917 and 1965..
         case 1920..1943:
         case 1950..1956:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Port
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(169)  ,UO+82800  ,3600 ,"S"   }),  // Jun 17
                     ({FIXED(306)  ,UO+0      ,0    ,""    })});// Nov 1
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+82800  ,3600 ,"S"   }),  // Mar 1
                     ({FIXED(287)  ,UO+82800  ,0    ,""    })});// Oct 14
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(60)   ,UO+82800  ,3600 ,"S"   }),  // Feb 29
                     ({FIXED(288)  ,UO+82800  ,0    ,""    })});// Oct 14
         case 1917:
         case 1919:
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(59)   ,UO+82800  ,3600 ,"S"   }),  // Feb 28
                     ({FIXED(287)  ,UO+82800  ,0    ,""    })});// Oct 14
         case 1924:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 16
                     ({FIXED(288)  ,UO+82800  ,0    ,""    })});// Oct 14
         case 1926:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 17
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1927:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+82800  ,3600 ,"S"   }),  // Apr 9
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1928:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 14
                     ({LDAY (281,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1929:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(110)  ,UO+82800  ,3600 ,"S"   }),  // Apr 20
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1932:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+82800  ,3600 ,"S"   }),  // Apr 2
                     ({LDAY (281,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1934:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(97)   ,UO+82800  ,3600 ,"S"   }),  // Apr 7
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1935:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(89)   ,UO+82800  ,3600 ,"S"   }),  // Mar 30
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1931:
         case 1936:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(108)  ,UO+82800  ,3600 ,"S"   }),  // Apr 18
                     ({LDAYL(280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1937:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+82800  ,3600 ,"S"   }),  // Apr 3
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1938:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+82800  ,3600 ,"S"   }),  // Mar 26
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(322)  ,UO+82800  ,0    ,""    })});// Nov 18
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(55)   ,UO+82800  ,3600 ,"S"   }),  // Feb 24
                     ({FIXED(279)  ,UO+82800  ,0    ,""    })});// Oct 5
         case 1941:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(95)   ,UO+82800  ,3600 ,"S"   }),  // Apr 5
                     ({FIXED(278)  ,UO+82800  ,0    ,""    })});// Oct 5
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (73,6) ,UO+82800  ,3600 ,"S"   }),  // Mar Sat>=8
                     ({FIXED(115)  ,UO+79200  ,7200 ,"M"   }),  // Apr 25
                     ({FIXED(227)  ,UO+79200  ,3600 ,"S"   }),  // Aug 15
                     ({LDAY (303,6),UO+82800  ,0    ,""    })});// Oct Sat>=24
         case 1943:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (73,6) ,UO+82800  ,3600 ,"S"   }),  // Mar Sat>=8
                     ({FIXED(107)  ,UO+79200  ,7200 ,"M"   }),  // Apr 17
                     ({LDAY (243,6),UO+79200  ,3600 ,"S"   }),  // Aug Sat>=25
                     ({LDAY (303,6),UO+82800  ,0    ,""    })});// Oct Sat>=24
         case 1944..1945:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(73,6) ,UO+82800  ,3600 ,"S"   }),  // Mar Sat>=8
                     ({LDAYL(117,6),UO+79200  ,7200 ,"M"   }),  // Apr Sat>=21
                     ({LDAYL(243,6),UO+79200  ,3600 ,"S"   }),  // Aug Sat>=25
                     ({LDAYL(303,6),UO+82800  ,0    ,""    })});// Oct Sat>=24
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,6) ,UO+82800  ,3600 ,"S"   }),  // Apr Sat>=1
                     ({LDAY (280,6),UO+82800  ,0    ,""    })});// Oct Sat>=1
         case 1947..1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAYL(280,7),UO+7200   ,0    ,""    })});// Oct Sun>=1
         default: // ..1915:
         case 1922..1923:
         case 1925:
         case 1930:
         case 1933:
         case 1966..1976:
            return ({({0           ,0         ,0    ,""    })});
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(86)   ,UO+0      ,3600 ,"S"   }),  // Mar 27
                     ({FIXED(268)  ,UO+0      ,0    ,""    })});// Sep 25
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(274)  ,UO+0      ,0    ,""    })});// Oct 1
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAY (273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (91,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAY (274,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1981..1982:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+3600   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAY (273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,"S"   })});// Mar lastSun
         case 1984..:
            return ({({0           ,0         ,3600 ,"S"   })});
      }
   }
}

class Romania
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1932:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(142)  ,UO+0      ,3600 ,"S"   }),  // May 21
                     ({LDAY (281,7),UO+0      ,0    ,""    })});// Oct Sun>=1
         case 1933..1939:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(98,7) ,UO+0      ,3600 ,"S"   }),  // Apr Sun>=2
                     ({LDAYL(280,7),UO+0      ,0    ,""    })});// Oct Sun>=1
         case 1979:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(147)  ,UO+0      ,3600 ,"S"   }),  // May 27
                     ({LDAY (273,7),UO-3600   ,0    ,""    })});// Sep lastSun
         case 1980:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+82800  ,3600 ,"S"   }),  // Apr 5
                     ({LDAY (274,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1991..1993:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+0      ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+0      ,0    ,""    })});// Sep lastSun
         default: // ..1931 and 1994..
         case 1940..1978:
         case 1981..1990:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Spain
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(105)  ,UO+82800  ,3600 ,"S"   }),  // Apr 15
                     ({FIXED(279)  ,UO+86400  ,0    ,""    })});// Oct 6
         case 1919:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(96)   ,UO+82800  ,3600 ,"S"   }),  // Apr 6
                     ({FIXED(279)  ,UO+86400  ,0    ,""    })});// Oct 6
         case 1924:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 16
                     ({FIXED(278)  ,UO+86400  ,0    ,""    })});// Oct 4
         case 1926:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(107)  ,UO+82800  ,3600 ,"S"   }),  // Apr 17
                     ({LDAY (280,6),UO+86400  ,0    ,""    })});// Oct Sat>=1
         case 1927:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(99)   ,UO+82800  ,3600 ,"S"   }),  // Apr 9
                     ({LDAY (280,6),UO+86400  ,0    ,""    })});// Oct Sat>=1
         case 1928:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(106)  ,UO+0      ,3600 ,"S"   }),  // Apr 15
                     ({LDAY (281,6),UO+86400  ,0    ,""    })});// Oct Sat>=1
         case 1929:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(110)  ,UO+82800  ,3600 ,"S"   }),  // Apr 20
                     ({LDAY (280,6),UO+86400  ,0    ,""    })});// Oct Sat>=1
         case 1937:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(167)  ,UO+82800  ,3600 ,"S"   }),  // Jun 16
                     ({FIXED(275)  ,UO+86400  ,0    ,""    })});// Oct 2
         case 1938:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+82800  ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(120)  ,UO+79200  ,7200 ,"M"   }),  // Apr 30
                     ({FIXED(275)  ,UO+79200  ,3600 ,"S"   })});// Oct 2
         case 1939:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(280)  ,UO+86400  ,0    ,""    })});// Oct 7
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+82800  ,3600 ,"S"   }),  // May 2
                     ({FIXED(244)  ,UO+0      ,0    ,""    })});// Sep 1
         case 1943..1944:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(109,6),UO+82800  ,3600 ,"S"   }),  // Apr Sat>=13
                     ({LDAYL(280,7),UO+0      ,0    ,""    })});// Oct Sun>=1
         case 1945..1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (109,6),UO+82800  ,3600 ,"S"   }),  // Apr Sat>=13
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(120)  ,UO+82800  ,3600 ,"S"   }),  // Apr 30
                     ({FIXED(275)  ,UO+0      ,0    ,""    })});// Oct 2
         case 1974..1975:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (108,6),UO+82800  ,3600 ,"S"   }),  // Apr Sat>=12
                     ({LDAY (280,7),UO+0      ,0    ,""    })});// Oct Sun>=1
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(87)   ,UO+82800  ,3600 ,"S"   }),  // Mar 27
                     ({LDAY (274,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+82800  ,3600 ,"S"   }),  // Apr 2
                     ({LDAY (273,7),UO+0      ,0    ,""    })});// Sep lastSun
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(92)   ,UO+7200   ,3600 ,"S"   }),  // Apr 2
                     ({FIXED(274)  ,UO+7200   ,0    ,""    })});// Oct 1
         default: // ..1917 and 1979..
         case 1920..1923:
         case 1925:
         case 1930..1936:
         case 1940..1941:
         case 1947..1948:
         case 1950..1973:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class SpainAfrica
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1967:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(154)  ,UO+43200  ,3600 ,"S"   }),  // Jun 3
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(175)  ,UO+0      ,3600 ,"S"   }),  // Jun 24
                     ({FIXED(244)  ,UO-3600   ,0    ,""    })});// Sep 1
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(122)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(214)  ,UO-3600   ,0    ,""    })});// Aug 1
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIXED(271)  ,UO-3600   ,0    ,""    })});// Sep 28
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({FIXED(216)  ,UO-3600   ,0    ,""    })});// Aug 4
         default: // ..1966 and 1979..
         case 1968..1973:
         case 1975:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Swiss
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1941..1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (127,1),UO+3600   ,3600 ,"S"   }),  // May Mon>=1
                     ({LDAY (280,1),UO+3600   ,0    ,""    })});// Oct Mon>=1
         default: // ..1940 and 1943..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Turkey
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(88)   ,UO+0      ,3600 ,"S"   }),  // Mar 28
                     ({FIXED(299)  ,UO-3600   ,0    ,""    })});// Oct 25
         case 1921:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(93)   ,UO+0      ,3600 ,"S"   }),  // Apr 3
                     ({FIXED(276)  ,UO-3600   ,0    ,""    })});// Oct 3
         case 1922:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(85)   ,UO+0      ,3600 ,"S"   }),  // Mar 26
                     ({FIXED(281)  ,UO-3600   ,0    ,""    })});// Oct 8
         case 1924:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(134)  ,UO+0      ,3600 ,"S"   }),  // May 13
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1916:
         case 1925:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(121)  ,UO+0      ,3600 ,"S"   }),  // May 1
                     ({FIX_L(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1940:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(183)  ,UO+0      ,3600 ,"S"   }),  // Jul 1
                     ({FIXED(280)  ,UO-3600   ,0    ,""    }),  // Oct 6
                     ({FIXED(336)  ,UO+0      ,3600 ,"S"   })});// Dec 1
         case 1941:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(264)  ,UO-3600   ,0    ,""    })});// Sep 21
         case 1942:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(91)   ,UO+0      ,3600 ,"S"   })});// Apr 1
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"S"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(281)  ,UO-3600   ,0    ,""    })});// Oct 8
         case 1946:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,"S"   }),  // Jun 1
                     ({FIXED(274)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1947..1948:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(112,7),UO+0      ,3600 ,"S"   }),  // Apr Sun>=16
                     ({LDAYL(281,7),UO-3600   ,0    ,""    })});// Oct Sun>=2
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(100)  ,UO+0      ,3600 ,"S"   }),  // Apr 10
                     ({LDAY (281,7),UO-3600   ,0    ,""    })});// Oct Sun>=2
         case 1950:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(106)  ,UO+0      ,3600 ,"S"   }),  // Apr 16
                     ({LDAY (281,7),UO-3600   ,0    ,""    })});// Oct Sun>=2
         case 1951:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(112)  ,UO+0      ,3600 ,"S"   }),  // Apr 22
                     ({LDAY (281,7),UO-3600   ,0    ,""    })});// Oct Sun>=2
         case 1962:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(196)  ,UO+0      ,3600 ,"S"   })});// Jul 15
         case 1963:
            return ({({0           ,0         ,3600 ,"S"   }),  
                     ({FIXED(303)  ,UO-3600   ,0    ,""    })});// Oct 30
         case 1964:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(136)  ,UO+0      ,3600 ,"S"   }),  // May 15
                     ({FIXED(275)  ,UO-3600   ,0    ,""    })});// Oct 1
         case 1973:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(154)  ,UO+3600   ,3600 ,"S"   }),  // Jun 3
                     ({LDAY (310,7),UO+3600   ,0    ,""    })});// Oct Sun>=31
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(90)   ,UO+7200   ,3600 ,"S"   }),  // Mar 31
                     ({LDAY (310,7),UO+3600   ,0    ,""    })});// Oct Sun>=31
         case 1975:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(81)   ,UO+7200   ,3600 ,"S"   }),  // Mar 22
                     ({LDAY (310,7),UO+3600   ,0    ,""    })});// Oct Sun>=31
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(81)   ,UO+7200   ,3600 ,"S"   }),  // Mar 21
                     ({LDAY (311,7),UO+3600   ,0    ,""    })});// Oct Sun>=31
         case 1977:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({LDAY (294,7),UO+3600   ,0    ,""    })});// Oct Sun>=15
         case 1978:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (97,7) ,UO+7200   ,3600 ,"S"   }),  // Apr Sun>=1
                     ({FIXED(180)  ,UO-3600   ,0    ,""    }),  // Jun 29
                     ({LDAY (294,7),UO+7200   ,0    ,""    })});// Oct Sun>=15
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(212)  ,UO+7200   ,3600 ,"S"   }),  // Jul 31
                     ({FIXED(275)  ,UO+3600   ,0    ,""    })});// Oct 2
         case 1985:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(110)  ,UO+3600   ,3600 ,"S"   }),  // Apr 20
                     ({FIXED(271)  ,UO+3600   ,0    ,""    })});// Sep 28
         case 1994:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(79)   ,UO+3600   ,3600 ,"S"   }),  // Mar 20
                     ({LDAY (273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1986..1993:
         case 1995:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+3600   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,""    })});// Sep lastSun
         case 1996..2006:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAYL(90,7) ,UO+3600   ,3600 ,"S"   }),  // Mar lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,""    })});// Oct lastSun
         default: // ..1915 and 2007..
         case 1917..1919:
         case 1923:
         case 1926..1939:
         case 1952..1961:
         case 1965..1972:
         case 1979..1982:
         case 1984:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class US
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918..1919:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,"D"   }),  // Mar lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({FIXED(273)  ,UO+3600   ,0    ,"S"   })});// Sep 30
         default: // ..1917:
         case 1920..1941:
         case 1946..1966:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1974:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(6)    ,UO+7200   ,3600 ,"D"   }),  // Jan 6
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1975:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (59,7) ,UO+7200   ,3600 ,"D"   }),  // Feb lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1967..1973:
         case 1976..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1987..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 2007..:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+7200   ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO+3600   ,0    ,"S"   })});// Nov Sun>=1
      }
   }
}

class NYC
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (91,7) ,UO+7200   ,3600 ,"D"   }),  // Mar lastSun
                     ({LDAY (305,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1921..1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1955..1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1919 and 1967..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Chicago
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(165)  ,UO+7200   ,3600 ,"D"   }),  // Jun 13
                     ({LDAY (305,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1921:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,"D"   }),  // Mar lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1922..1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1955..1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1919 and 1967..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Denver
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1920:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (91,7) ,UO+7200   ,3600 ,"D"   }),  // Mar lastSun
                     ({LDAY (305,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1921:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (90,7) ,UO+7200   ,3600 ,"D"   }),  // Mar lastSun
                     ({FIXED(142)  ,UO+3600   ,0    ,"S"   })});// May 22
         case 1965..1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1919 and 1967..
         case 1922..1964:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class CA
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1948:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(74)   ,UO+7260   ,3600 ,"D"   })});// Mar 14
         case 1949:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(1)    ,UO+3600   ,0    ,"S"   })});// Jan 1
         case 1950..1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+3600   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1962..1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+3600   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1947 and 1967..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Indianapolis
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1941:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(173)  ,UO+7200   ,3600 ,"D"   }),  // Jun 22
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1942..1945:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(273,7),UO+7200   ,0    ,"S"   })});// Sep lastSun
         case 1946..1954:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         default: // ..1940 and 1955..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Marengo
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1951:
         case 1954..1960:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         default: // ..1950 and 1961..
         case 1952..1953:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Vincennes
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1955:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1946:
         case 1953..1954:
         case 1956..1959:
         case 1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1960:
         case 1962..1963:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1945 and 1964..
         case 1947..1952:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Perry
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1955:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(121)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1956..1960:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1961..1963:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1954 and 1964..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Pike
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1955:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(121)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1956..1960:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1961..1964:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1954 and 1965..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Starke
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1947..1954:
         case 1957..1958:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1955..1956:
         case 1959..1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1946 and 1962..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Pulaski
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1955..1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1946..1954:
         case 1957..1960:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         default: // ..1945 and 1961..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Louisville
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1921:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(121)  ,UO+7200   ,3600 ,"D"   }),  // May 1
                     ({FIXED(244)  ,UO+3600   ,0    ,"S"   })});// Sep 1
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+60     ,3600 ,"D"   }),  // Apr lastSun
                     ({FIXED(153)  ,UO+3600   ,0    ,"S"   })});// Jun 2
         case 1941:
         case 1950..1955:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1956..1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1920 and 1962..
         case 1922..1940:
         case 1942..1945:
         case 1947..1949:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Detroit
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1948:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (121,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (274,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         default: // ..1947 and 1949..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Menominee
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1945 and 1967..
         case 1947..1965:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Canada
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(104)  ,UO+7200   ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(300)  ,UO+3600   ,0    ,"S"   })});// Oct 27
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({FIXED(273)  ,UO+3600   ,0    ,"S"   })});// Sep 30
         default: // ..1917:
         case 1919..1941:
         case 1946..1973:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1974..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1987..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 2007..:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+7200   ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO+3600   ,0    ,"S"   })});// Nov Sun>=1
      }
   }
}

class StJohns
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1917:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(98)   ,UO+7200   ,3600 ,"D"   }),  // Apr 8
                     ({FIXED(260)  ,UO+3600   ,0    ,"S"   })});// Sep 17
         case 1919:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(125)  ,UO+82800  ,3600 ,"D"   }),  // May 5
                     ({FIXED(224)  ,UO+79200  ,0    ,"S"   })});// Aug 12
         case 1920..1935:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+82800  ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(304,7),UO+79200  ,0    ,"S"   })});// Oct lastSun
         case 1936..1941:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(135,1),UO+0      ,3600 ,"D"   }),  // May Mon>=9
                     ({LDAYL(281,1),UO-3600   ,0    ,"S"   })});// Oct Mon>=2
         default: // ..1916:
         case 1918:
         case 1942..1945:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1946..1950:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(134,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=8
                     ({LDAYL(281,7),UO+3600   ,0    ,"S"   })});// Oct Sun>=2
         case 1951..1959:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1960..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1988:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (98,7) ,UO+60     ,7200 ,"DD"  }),  // Apr Sun>=1
                     ({LDAY (305,7),UO-7140   ,0    ,"S"   })});// Oct lastSun
         case 1987:
         case 1989..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+60     ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO-3540   ,0    ,"S"   })});// Oct lastSun
         case 2007..2010:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+60     ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO-3540   ,0    ,"S"   })});// Nov Sun>=1
         case 2011:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (73,7) ,UO+60     ,3600 ,"D"   })});// Mar Sun>=8
         case 2012..:
            return ({({0           ,0         ,3600 ,"D"   })});
      }
   }
}

class Halifax
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(92)   ,UO+0      ,3600 ,"D"   }),  // Apr 1
                     ({FIXED(275)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         case 1920:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(130)  ,UO+0      ,3600 ,"D"   }),  // May 9
                     ({FIXED(242)  ,UO-3600   ,0    ,"S"   })});// Aug 29
         case 1921:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(126)  ,UO+0      ,3600 ,"D"   }),  // May 6
                     ({FIXED(248)  ,UO-3600   ,0    ,"S"   })});// Sep 5
         case 1922:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"D"   }),  // Apr 30
                     ({FIXED(248)  ,UO-3600   ,0    ,"S"   })});// Sep 5
         case 1923:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({FIXED(247)  ,UO-3600   ,0    ,"S"   })});// Sep 4
         case 1924:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (128,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({FIXED(259)  ,UO-3600   ,0    ,"S"   })});// Sep 15
         case 1925:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({FIXED(271)  ,UO-3600   ,0    ,"S"   })});// Sep 28
         case 1926:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(136)  ,UO+0      ,3600 ,"D"   }),  // May 16
                     ({FIXED(256)  ,UO-3600   ,0    ,"S"   })});// Sep 13
         case 1927:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(121)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({FIXED(269)  ,UO-3600   ,0    ,"S"   })});// Sep 26
         case 1928:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (135,7),UO+0      ,3600 ,"D"   }),  // May Sun>=8
                     ({FIXED(253)  ,UO-3600   ,0    ,"S"   })});// Sep 9
         case 1929:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (134,7),UO+0      ,3600 ,"D"   }),  // May Sun>=8
                     ({FIXED(246)  ,UO-3600   ,0    ,"S"   })});// Sep 3
         case 1930:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (134,7),UO+0      ,3600 ,"D"   }),  // May Sun>=8
                     ({FIXED(258)  ,UO-3600   ,0    ,"S"   })});// Sep 15
         case 1931:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (134,7),UO+0      ,3600 ,"D"   }),  // May Sun>=8
                     ({LDAY (273,1),UO-3600   ,0    ,"S"   })});// Sep Mon>=24
         case 1932:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(122)  ,UO+0      ,3600 ,"D"   }),  // May 1
                     ({LDAY (274,1),UO-3600   ,0    ,"S"   })});// Sep Mon>=24
         case 1933:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"D"   }),  // Apr 30
                     ({FIXED(275)  ,UO-3600   ,0    ,"S"   })});// Oct 2
         case 1934:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(140)  ,UO+0      ,3600 ,"D"   }),  // May 20
                     ({FIXED(259)  ,UO-3600   ,0    ,"S"   })});// Sep 16
         case 1935:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,"D"   }),  // Jun 2
                     ({FIXED(273)  ,UO-3600   ,0    ,"S"   })});// Sep 30
         case 1936:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(153)  ,UO+0      ,3600 ,"D"   }),  // Jun 1
                     ({FIXED(258)  ,UO-3600   ,0    ,"S"   })});// Sep 14
         case 1939:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(148)  ,UO+0      ,3600 ,"D"   }),  // May 28
                     ({LDAY (273,1),UO-3600   ,0    ,"S"   })});// Sep Mon>=24
         case 1937..1938:
         case 1940..1941:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(273,1),UO-3600   ,0    ,"S"   })});// Sep Mon>=24
         case 1946..1949:
         case 1951..1954:
         case 1956..1959:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1962..1973:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1915 and 1974..
         case 1917..1919:
         case 1942..1945:
         case 1950:
         case 1955:
         case 1960..1961:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Moncton
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1933..1935:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (165,7),UO+3600   ,3600 ,"D"   }),  // Jun Sun>=8
                     ({LDAY (257,7),UO+0      ,0    ,"S"   })});// Sep Sun>=8
         case 1936..1938:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(158,7),UO+3600   ,3600 ,"D"   }),  // Jun Sun>=1
                     ({LDAYL(250,7),UO+0      ,0    ,"S"   })});// Sep Sun>=1
         case 1939:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(147)  ,UO+3600   ,3600 ,"D"   }),  // May 27
                     ({LDAY (270,6),UO+0      ,0    ,"S"   })});// Sep Sat>=21
         case 1940:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(140)  ,UO+3600   ,3600 ,"D"   }),  // May 19
                     ({LDAY (271,6),UO+0      ,0    ,"S"   })});// Sep Sat>=21
         case 1941:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(124)  ,UO+3600   ,3600 ,"D"   }),  // May 4
                     ({LDAY (270,6),UO+0      ,0    ,"S"   })});// Sep Sat>=21
         case 1946..1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1957..1972:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1993..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+60     ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO-3540   ,0    ,"S"   })});// Oct lastSun
         default: // ..1932 and 2007..
         case 1942..1945:
         case 1973..1992:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Toronto
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1919:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(89)   ,UO+84600  ,3600 ,"D"   }),  // Mar 30
                     ({FIXED(299)  ,UO-3600   ,0    ,"S"   })});// Oct 26
         case 1920:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(123)  ,UO+7200   ,3600 ,"D"   }),  // May 2
                     ({FIXED(270)  ,UO-3600   ,0    ,"S"   })});// Sep 26
         case 1921:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(135)  ,UO+7200   ,3600 ,"D"   }),  // May 15
                     ({FIXED(258)  ,UO+3600   ,0    ,"S"   })});// Sep 15
         case 1922..1923:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (134,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=8
                     ({LDAY (264,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=15
         case 1924..1926:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(264,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=15
         case 1927:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (127,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAY (274,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=25
         case 1928..1937:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(121,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=25
                     ({LDAYL(274,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=25
         case 1940:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (121,7),UO+7200   ,3600 ,"D"   })});// Apr lastSun
         case 1941..1944:
            return ({({0           ,0         ,3600 ,"D"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1949..1950:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (334,7),UO+3600   ,0    ,"S"   })});// Nov lastSun
         case 1938..1939:
         case 1946..1948:
         case 1951..1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1957..1973:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1918 and 1974..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Winn
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1916:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(114)  ,UO+0      ,3600 ,"D"   }),  // Apr 23
                     ({FIXED(261)  ,UO-3600   ,0    ,"S"   })});// Sep 17
         case 1918:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(104)  ,UO+7200   ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(300)  ,UO+3600   ,0    ,"S"   })});// Oct 27
         case 1937:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(136)  ,UO+7200   ,3600 ,"D"   }),  // May 16
                     ({FIXED(269)  ,UO+3600   ,0    ,"S"   })});// Sep 26
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(132)  ,UO+7200   ,3600 ,"D"   }),  // May 12
                     ({FIXED(286)  ,UO+3600   ,0    ,"S"   })});// Oct 13
         case 1950:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(121)  ,UO+7200   ,3600 ,"D"   }),  // May 1
                     ({FIXED(273)  ,UO+3600   ,0    ,"S"   })});// Sep 30
         case 1959:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1947..1949:
         case 1951..1958:
         case 1960:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1963:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({FIXED(265)  ,UO+3600   ,0    ,"S"   })});// Sep 22
         case 1966..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+7200   ,0    ,"S"   })});// Oct lastSun
         case 1987..2005:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+7200   ,0    ,"S"   })});// Oct lastSun
         default: // ..1915 and 2006..
         case 1917:
         case 1919..1936:
         case 1938..1941:
         case 1961..1962:
         case 1964..1965:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Regina
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(104)  ,UO+7200   ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(300)  ,UO+3600   ,0    ,"S"   })});// Oct 27
         case 1930..1934:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(280,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=1
         case 1938:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (104,7),UO+0      ,3600 ,"D"   }),  // Apr Sun>=8
                     ({LDAY (280,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=1
         case 1937:
         case 1939..1941:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(104,7),UO+0      ,3600 ,"D"   }),  // Apr Sun>=8
                     ({LDAYL(287,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (104,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=8
                     ({LDAY (287,7),UO+3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1947..1957:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1959:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1917 and 1960..
         case 1919..1929:
         case 1935..1936:
         case 1958:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Swift
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1957:
         case 1959:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1960..1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         default: // ..1956 and 1962..
         case 1958:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Edm
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (104,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=8
                     ({FIXED(300)  ,UO+3600   ,0    ,"S"   })});// Oct 27
         case 1919:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (104,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=8
                     ({FIXED(147)  ,UO+3600   ,0    ,"S"   })});// May 27
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1921..1923:
         case 1947:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1920:
         case 1972..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1987..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(304,7),UO+7200   ,0    ,"S"   })});// Oct lastSun
         default: // ..1917 and 2007..
         case 1924..1941:
         case 1946:
         case 1948..1971:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Vanc
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(104)  ,UO+7200   ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(300)  ,UO+3600   ,0    ,"S"   })});// Oct 27
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({FIXED(273)  ,UO+3600   ,0    ,"S"   })});// Sep 30
         case 1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({FIXED(272)  ,UO+3600   ,0    ,"S"   })});// Sep 29
         case 1947..1961:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1962..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1987..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(304,7),UO+7200   ,0    ,"S"   })});// Oct lastSun
         default: // ..1917 and 2007..
         case 1919..1941:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class NT_YK
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1918:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(104)  ,UO+7200   ,3600 ,"D"   }),  // Apr 14
                     ({FIXED(300)  ,UO+3600   ,0    ,"S"   })});// Oct 27
         case 1919:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(145)  ,UO+7200   ,3600 ,"D"   }),  // May 25
                     ({FIXED(305)  ,UO-3600   ,0    ,"S"   })});// Nov 1
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(40)   ,UO+7200   ,3600 ,"W"   })});// Feb 9
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"W"   })});
         case 1945:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(226)  ,82800     ,3600 ,"P"   }),  // Aug 14
                     ({FIXED(273)  ,UO+3600   ,0    ,"S"   })});// Sep 30
         case 1972..1986:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+7200   ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1987..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1917 and 2007..
         case 1920..1941:
         case 1946..1971:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Yukon
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1965:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({LDAY (120,7),UO+0      ,7200 ,"DD"  }),  // Apr lastSun
                     ({LDAY (304,7),UO+0      ,0    ,"S"   })});// Oct lastSun
         default: // ..1964 and 1966..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Mexico
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1931:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(121)  ,UO+82800  ,3600 ,"D"   }),  // May 1
                     ({FIXED(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         case 1939:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(36)   ,UO+0      ,3600 ,"D"   }),  // Feb 5
                     ({FIXED(176)  ,UO-3600   ,0    ,"S"   })});// Jun 25
         case 1940:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(344)  ,UO+0      ,3600 ,"D"   })});// Dec 9
         case 1941:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(91)   ,UO-3600   ,0    ,"S"   })});// Apr 1
         case 1943:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(350)  ,UO+0      ,3600 ,"W"   })});// Dec 16
         case 1944:
            return ({({0           ,0         ,3600 ,"W"   }),  
                     ({FIXED(122)  ,UO-3600   ,0    ,"S"   })});// May 1
         case 1950:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(43)   ,UO+0      ,3600 ,"D"   }),  // Feb 12
                     ({FIXED(211)  ,UO-3600   ,0    ,"S"   })});// Jul 30
         case 2001:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (127,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAY (273,7),UO+3600   ,0    ,"S"   })});// Sep lastSun
         case 1996..2000:
         case 2002..2022:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+7200   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1930 and 2023..
         case 1932..1938:
         case 1942:
         case 1945..1949:
         case 1951..1995:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Barb
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(109)  ,18000     ,3600 ,"D"   }),  // Apr 19
                     ({FIXED(243)  ,21600     ,0    ,"S"   })});// Aug 31
         case 1943:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(122)  ,18000     ,3600 ,"D"   }),  // May 2
                     ({FIXED(248)  ,21600     ,0    ,"S"   })});// Sep 5
         case 1944:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(101)  ,18000     ,1800 ,""    }),  // Apr 10
                     ({FIXED(254)  ,21600     ,0    ,"S"   })});// Sep 10
         case 1977:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(163)  ,UO+7200   ,3600 ,"D"   }),  // Jun 12
                     ({LDAY (280,7),UO+3600   ,0    ,"S"   })});// Oct Sun>=1
         case 1978:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (111,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=15
                     ({LDAY (280,7),UO+3600   ,0    ,"S"   })});// Oct Sun>=1
         case 1979:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (111,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=15
                     ({FIXED(273)  ,UO+3600   ,0    ,"S"   })});// Sep 30
         case 1980:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (112,7),UO+7200   ,3600 ,"D"   }),  // Apr Sun>=15
                     ({FIXED(269)  ,UO+3600   ,0    ,"S"   })});// Sep 25
         default: // ..1941 and 1981..
         case 1945..1976:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Belize
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1942:
            return ({({0           ,0         ,1800 ,"-0530"}),  
                     ({LDAY (45,6) ,UO+84600  ,0    ,"CST" }),  // Feb Sat>=8
                     ({FIXED(178)  ,UO+86400  ,3600 ,"CWT" })});// Jun 27
         case 1943..1944:
            return ({({0           ,0         ,3600 ,"CWT" })});
         case 1945:
            return ({({0           ,0         ,3600 ,"CWT" }),  
                     ({FIXED(226)  ,82800     ,3600 ,"CPT" }),  // Aug 14
                     ({FIXED(349)  ,UO+82800  ,0    ,"CST" })});// Dec 15
         case 1918:
         case 1947:
            return ({({0           ,0         ,0    ,"CST" }),  
                     ({LDAY (280,6),UO+86400  ,1800 ,"-0530"})});// Oct Sat>=1
         case 1919..1941:
         case 1948..1967:
            return ({({0           ,0         ,1800 ,"-0530"}),  
                     ({LDAYL(45,6) ,UO+84600  ,0    ,"CST" }),  // Feb Sat>=8
                     ({LDAYL(280,6),UO+86400  ,1800 ,"-0530"})});// Oct Sat>=1
         case 1968:
            return ({({0           ,0         ,1800 ,"-0530"}),  
                     ({LDAY (46,6) ,UO+84600  ,0    ,"CST" })});// Feb Sat>=8
         case 1973:
            return ({({0           ,0         ,0    ,"CST" }),  
                     ({FIXED(339)  ,UO+0      ,3600 ,"CDT" })});// Dec 5
         case 1974:
            return ({({0           ,0         ,3600 ,"CDT" }),  
                     ({FIXED(40)   ,UO-3600   ,0    ,"CST" })});// Feb 9
         case 1982:
            return ({({0           ,0         ,0    ,"CST" }),  
                     ({FIXED(352)  ,UO+0      ,3600 ,"CDT" })});// Dec 18
         case 1983:
            return ({({0           ,0         ,3600 ,"CDT" }),  
                     ({FIXED(43)   ,UO-3600   ,0    ,"CST" })});// Feb 12
         default: // ..1917 and 1984..
         case 1946:
         case 1969..1972:
         case 1975..1981:
            return ({({0           ,0         ,0    ,"CST" })});
      }
   }
}

class Bermuda
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1916:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1917:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(95)   ,UO+86400  ,3600 ,""    }),  // Apr 5
                     ({FIXED(273)  ,UO+82800  ,0    ,""    })});// Sep 30
         case 1918:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(103)  ,UO+86400  ,3600 ,""    }),  // Apr 13
                     ({FIXED(258)  ,UO+82800  ,0    ,"S"   })});// Sep 15
         case 1942:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(11)   ,UO+7200   ,3600 ,"D"   }),  // Jan 11
                     ({FIXED(291)  ,UO+3600   ,0    ,"S"   })});// Oct 18
         case 1943:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(80)   ,UO+7200   ,3600 ,"D"   }),  // Mar 21
                     ({FIXED(304)  ,UO+3600   ,0    ,"S"   })});// Oct 31
         case 1944..1945:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+7200   ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO+3600   ,0    ,"S"   })});// Nov Sun>=1
         case 1947:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (141,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=15
                     ({LDAY (257,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=8
         case 1948..1952:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(148,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=22
                     ({LDAYL(250,7),UO+3600   ,0    ,"S"   })});// Sep Sun>=1
         case 1956:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (149,7),UO+7200   ,3600 ,"D"   }),  // May Sun>=22
                     ({LDAY (305,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 1919..1941:
         case 1946:
         case 1953..1955:
         case 1957..:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class CR
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1979..1980:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(59,7) ,UO+0      ,3600 ,"D"   }),  // Feb lastSun
                     ({LDAYL(158,7),UO-3600   ,0    ,"S"   })});// Jun Sun>=1
         case 1991:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (21,6) ,UO+0      ,3600 ,"D"   }),  // Jan Sat>=15
                     ({FIXED(182)  ,UO-3600   ,0    ,"S"   })});// Jul 1
         case 1992:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (21,6) ,UO+0      ,3600 ,"D"   }),  // Jan Sat>=15
                     ({FIXED(75)   ,UO-3600   ,0    ,"S"   })});// Mar 15
         default: // ..1978 and 1993..
         case 1981..1990:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Cuba
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1928:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(162)  ,UO+0      ,3600 ,"D"   }),  // Jun 10
                     ({FIXED(284)  ,UO-3600   ,0    ,"S"   })});// Oct 10
         case 1940..1942:
         case 1945..1946:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(158,7),UO+0      ,3600 ,"D"   }),  // Jun Sun>=1
                     ({LDAYL(250,7),UO-3600   ,0    ,"S"   })});// Sep Sun>=1
         default: // ..1927:
         case 1929..1939:
         case 1943..1944:
         case 1947..1964:
            return ({({0           ,0         ,0    ,"S"   })});
         case 1965:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(152)  ,UO+0      ,3600 ,"D"   }),  // Jun 1
                     ({FIXED(273)  ,UO-3600   ,0    ,"S"   })});// Sep 30
         case 1966:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(149)  ,UO+0      ,3600 ,"D"   }),  // May 29
                     ({FIXED(275)  ,UO-3600   ,0    ,"S"   })});// Oct 2
         case 1967:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(98)   ,UO+0      ,3600 ,"D"   }),  // Apr 8
                     ({LDAY (257,7),UO-3600   ,0    ,"S"   })});// Sep Sun>=8
         case 1968:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(105)  ,UO+0      ,3600 ,"D"   }),  // Apr 14
                     ({LDAY (258,7),UO-3600   ,0    ,"S"   })});// Sep Sun>=8
         case 1972..1974:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+0      ,3600 ,"D"   }),  // Apr lastSun
                     ({FIX_L(281)  ,UO-3600   ,0    ,"S"   })});// Oct 8
         case 1969..1971:
         case 1975..1977:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+0      ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO-3600   ,0    ,"S"   })});// Oct lastSun
         case 1978:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(127)  ,UO+0      ,3600 ,"D"   }),  // May 7
                     ({LDAY (287,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1979..1980:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(80,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=15
                     ({LDAYL(287,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1981..1985:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(131,7),UO+0      ,3600 ,"D"   }),  // May Sun>=5
                     ({LDAYL(287,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1986..1989:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(79,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=14
                     ({LDAYL(287,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1990:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAY (287,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=8
         case 1991..1995:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+0      ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(287,7),UO+0      ,0    ,"S"   })});// Oct Sun>=8
         case 1996:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (98,7) ,UO+0      ,3600 ,"D"   }),  // Apr Sun>=1
                     ({FIXED(280)  ,UO+0      ,0    ,"S"   })});// Oct 6
         case 1997:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"D"   }),  // Apr Sun>=1
                     ({FIXED(285)  ,UO+0      ,0    ,"S"   })});// Oct 12
         case 1998..1999:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (90,7) ,UO+0      ,3600 ,"D"   }),  // Mar lastSun
                     ({LDAY (304,7),UO+0      ,0    ,"S"   })});// Oct lastSun
         case 2000..2003:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+0      ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+0      ,0    ,"S"   })});// Oct lastSun
         case 2004:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (91,7) ,UO+0      ,3600 ,"D"   })});// Mar lastSun
         case 2005:
            return ({({0           ,0         ,3600 ,"D"   })});
         case 2006:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({LDAY (304,7),UO+0      ,0    ,"S"   })});// Oct lastSun
         case 2008:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (81,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=15
                     ({LDAY (305,7),UO+0      ,0    ,"S"   })});// Oct lastSun
         case 2007:
         case 2009..2010:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (73,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAY (304,7),UO+0      ,0    ,"S"   })});// Oct lastSun
         case 2011:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (80,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=15
                     ({FIXED(317)  ,UO+0      ,0    ,"S"   })});// Nov 13
         case 2012:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(92)   ,UO+0      ,3600 ,"D"   }),  // Apr 1
                     ({LDAY (312,7),UO+0      ,0    ,"S"   })});// Nov Sun>=1
         case 2013..:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO+0      ,0    ,"S"   })});// Nov Sun>=1
      }
   }
}

class DR
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1966:
            return ({({0           ,0         ,0    ,"EST" }),  // ?
                     ({FIXED(303)  ,UO+0      ,3600 ,"EDT" })});// Oct 30
         case 1967:
            return ({({0           ,0         ,3600 ,"EDT" }),  
                     ({FIXED(59)   ,UO-3600   ,0    ,"EST" })});// Feb 28
         case 1969:
            return ({({0           ,0         ,0    ,"EST" }),  
                     ({LDAY (304,7),UO+0      ,1800 ,"-0430"})});// Oct lastSun
         case 1970:
            return ({({0           ,0         ,1800 ,"-0430"}),  
                     ({FIXED(52)   ,UO-1800   ,0    ,"EST" }),  // Feb 21
                     ({LDAY (304,7),UO+0      ,1800 ,"-0430"})});// Oct lastSun
         case 1971:
            return ({({0           ,0         ,1800 ,"-0430"}),  
                     ({FIXED(20)   ,UO-1800   ,0    ,"EST" }),  // Jan 20
                     ({LDAY (304,7),UO+0      ,1800 ,"-0430"})});// Oct lastSun
         case 1972..1973:
            return ({({0           ,0         ,1800 ,"-0430"}),  
                     ({FIXED(21)   ,UO-1800   ,0    ,"EST" }),  // Jan 21
                     ({LDAYL(304,7),UO+0      ,1800 ,"-0430"})});// Oct lastSun
         case 1974:
            return ({({0           ,0         ,1800 ,"-0430"}),  
                     ({FIXED(21)   ,UO-1800   ,0    ,"EST" })});// Jan 21
         default: // ..1965 and 1975..
         case 1968:
            return ({({0           ,0         ,0    ,"EST" })});
      }
   }
}

class Salv
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1987..1988:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(273,7),UO-3600   ,0    ,"S"   })});// Sep lastSun
         default: // ..1986 and 1989..
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Guat
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1973:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(329)  ,UO+0      ,3600 ,"D"   })});// Nov 25
         case 1974:
            return ({({0           ,0         ,3600 ,"D"   }),  
                     ({FIXED(55)   ,UO-3600   ,0    ,"S"   })});// Feb 24
         case 1983:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(141)  ,UO+0      ,3600 ,"D"   }),  // May 21
                     ({FIXED(265)  ,UO-3600   ,0    ,"S"   })});// Sep 22
         case 1991:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(82)   ,UO+0      ,3600 ,"D"   }),  // Mar 23
                     ({FIXED(250)  ,UO-3600   ,0    ,"S"   })});// Sep 7
         case 2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,UO+0      ,3600 ,"D"   }),  // Apr 30
                     ({FIXED(274)  ,UO-3600   ,0    ,"S"   })});// Oct 1
         default: // ..1972 and 2007..
         case 1975..1982:
         case 1984..1990:
         case 1992..2005:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Haiti
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1983:
            return ({({0           ,0         ,0    ,"S"   }),  // ?
                     ({FIXED(128)  ,UO+0      ,3600 ,"D"   }),  // May 8
                     ({LDAY (304,7),UO-3600   ,0    ,"S"   })});// Oct lastSun
         case 1984..1987:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(120,7),UO+0      ,3600 ,"D"   }),  // Apr lastSun
                     ({LDAYL(304,7),UO-3600   ,0    ,"S"   })});// Oct lastSun
         case 1988..1997:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(97,7) ,UO+3600   ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAYL(304,7),UO+3600   ,0    ,"S"   })});// Oct lastSun
         case 2005..2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (97,7) ,UO+0      ,3600 ,"D"   }),  // Apr Sun>=1
                     ({LDAY (304,7),UO-3600   ,0    ,"S"   })});// Oct lastSun
         default: // ..1982:
         case 1998..2004:
         case 2007..2011:
         case 2016:
            return ({({0           ,0         ,0    ,"S"   })});
         case 2012..2015:
         case 2017..:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(73,7) ,UO+7200   ,3600 ,"D"   }),  // Mar Sun>=8
                     ({LDAYL(311,7),UO+3600   ,0    ,"S"   })});// Nov Sun>=1
      }
   }
}

class Hond
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1987..1988:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAYL(273,7),UO-3600   ,0    ,"S"   })});// Sep lastSun
         case 2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAY (127,7),UO+0      ,3600 ,"D"   }),  // May Sun>=1
                     ({LDAY (219,1),UO-3600   ,0    ,"S"   })});// Aug Mon>=1
         default: // ..1986 and 2007..
         case 1989..2005:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Nic
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1979..1980:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({LDAYL(81,7) ,UO+0      ,3600 ,"D"   }),  // Mar Sun>=16
                     ({LDAYL(180,1),UO-3600   ,0    ,"S"   })});// Jun Mon>=23
         case 2005:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(100)  ,UO+0      ,3600 ,"D"   }),  // Apr 10
                     ({LDAY (280,7),UO-3600   ,0    ,"S"   })});// Oct Sun>=1
         case 2006:
            return ({({0           ,0         ,0    ,"S"   }),  
                     ({FIXED(120)  ,UO+7200   ,3600 ,"D"   }),  // Apr 30
                     ({LDAY (280,7),UO+0      ,0    ,"S"   })});// Oct Sun>=1
         default: // ..1978 and 2007..
         case 1981..2004:
            return ({({0           ,0         ,0    ,"S"   })});
      }
   }
}

class Arg
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1931:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIXED(288)  ,UO+0      ,3600 ,""    })});// Oct 15
         case 1932..1939:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(60)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIX_L(305)  ,UO+0      ,3600 ,""    })});// Nov 1
         case 1940:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(61)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIXED(183)  ,UO+0      ,3600 ,""    })});// Jul 1
         case 1941:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(166)  ,UO-3600   ,0    ,""    }),  // Jun 15
                     ({FIXED(288)  ,UO+0      ,3600 ,""    })});// Oct 15
         case 1943:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(213)  ,UO-3600   ,0    ,""    }),  // Aug 1
                     ({FIXED(288)  ,UO+0      ,3600 ,""    })});// Oct 15
         case 1946:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(60)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIXED(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1942:
         case 1944..1945:
         case 1947..1962:
            return ({({0           ,0         ,3600 ,""    })});
         case 1963:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(274)  ,UO-3600   ,0    ,""    }),  // Oct 1
                     ({FIXED(349)  ,UO+0      ,3600 ,""    })});// Dec 15
         case 1964..1966:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(60)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIX_L(288)  ,UO+0      ,3600 ,""    })});// Oct 15
         case 1967:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(92)   ,UO-3600   ,0    ,""    }),  // Apr 2
                     ({LDAY (280,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 1968:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (98,7) ,UO-3600   ,0    ,""    }),  // Apr Sun>=1
                     ({LDAY (281,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 1969:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (97,7) ,UO-3600   ,0    ,""    })});// Apr Sun>=1
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(23)   ,UO+0      ,3600 ,""    }),  // Jan 23
                     ({FIXED(121)  ,UO-3600   ,0    ,""    })});// May 1
         case 1930:
         case 1988:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIX_L(335)  ,UO+0      ,3600 ,""    })});// Dec 1
         case 1989..1992:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(66,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAYL(294,7),UO+0      ,3600 ,""    })});// Oct Sun>=15
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO-3600   ,0    ,""    })});// Mar Sun>=1
         case 1999:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (280,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 2000:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(63)   ,UO-3600   ,0    ,""    })});// Mar 3
         case 2007:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(364)  ,UO+0      ,3600 ,""    })});// Dec 30
         case 2008:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (81,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=15
                     ({LDAY (295,7),UO+0      ,3600 ,""    })});// Oct Sun>=15
         case 2009:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (80,7) ,UO-3600   ,0    ,""    })});// Mar Sun>=15
         default: // ..1929 and 2010..
         case 1970..1973:
         case 1975..1987:
         case 1994..1998:
         case 2001..2006:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class SanLuis
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 2007:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({LDAY (287,7),UO+0      ,3600 ,""    })});// Oct Sun>=8
         case 2008:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (74,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=8
                     ({LDAY (288,7),UO+0      ,3600 ,""    })});// Oct Sun>=8
         case 2009:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (73,7) ,UO-3600   ,0    ,""    })});// Mar Sun>=8
         default: // ..2006 and 2010..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Brazil
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1931:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(276)  ,UO+39600  ,3600 ,""    })});// Oct 3
         case 1932:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(92)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIXED(277)  ,UO+0      ,3600 ,""    })});// Oct 3
         case 1933:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(91)   ,UO-3600   ,0    ,""    })});// Apr 1
         case 1949:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(335)  ,UO+0      ,3600 ,""    })});// Dec 1
         case 1950:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(106)  ,UO+0      ,0    ,""    }),  // Apr 16
                     ({FIXED(335)  ,UO+0      ,3600 ,""    })});// Dec 1
         case 1951..1952:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIX_L(335)  ,UO+0      ,3600 ,""    })});// Dec 1
         case 1963:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(343)  ,UO+0      ,3600 ,""    })});// Dec 9
         case 1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(31)   ,UO+0      ,3600 ,""    }),  // Jan 31
                     ({FIXED(90)   ,UO-3600   ,0    ,""    }),  // Mar 31
                     ({FIXED(335)  ,UO+0      ,3600 ,""    })});// Dec 1
         case 1966..1967:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(60)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIXED(305)  ,UO+0      ,3600 ,""    })});// Nov 1
         case 1953:
         case 1964:
         case 1968:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(60)   ,UO-3600   ,0    ,""    })});// Mar 1
         case 1985:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(306)  ,UO+0      ,3600 ,""    })});// Nov 2
         case 1986:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(74)   ,UO-3600   ,0    ,""    }),  // Mar 15
                     ({FIXED(298)  ,UO+0      ,3600 ,""    })});// Oct 25
         case 1987:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(45)   ,UO-3600   ,0    ,""    }),  // Feb 14
                     ({FIXED(298)  ,UO+0      ,3600 ,""    })});// Oct 25
         case 1988:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(38)   ,UO-3600   ,0    ,""    }),  // Feb 7
                     ({FIXED(290)  ,UO+0      ,3600 ,""    })});// Oct 16
         case 1989:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(29)   ,UO-3600   ,0    ,""    }),  // Jan 29
                     ({FIXED(288)  ,UO+0      ,3600 ,""    })});// Oct 15
         case 1990:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(42)   ,UO-3600   ,0    ,""    }),  // Feb 11
                     ({FIXED(294)  ,UO+0      ,3600 ,""    })});// Oct 21
         case 1991:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(48)   ,UO-3600   ,0    ,""    }),  // Feb 17
                     ({FIXED(293)  ,UO+0      ,3600 ,""    })});// Oct 20
         case 1992:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(40)   ,UO-3600   ,0    ,""    }),  // Feb 9
                     ({FIXED(299)  ,UO+0      ,3600 ,""    })});// Oct 25
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(31)   ,UO-3600   ,0    ,""    }),  // Jan 31
                     ({LDAY (290,7),UO+0      ,3600 ,""    })});// Oct Sun>=11
         case 1994..1995:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({LDAY (290,7),UO+0      ,3600 ,""    })});// Oct Sun>=11
         case 1996:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(42)   ,UO-3600   ,0    ,""    }),  // Feb 11
                     ({FIXED(280)  ,UO+0      ,3600 ,""    })});// Oct 6
         case 1997:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(47)   ,UO-3600   ,0    ,""    }),  // Feb 16
                     ({FIXED(279)  ,UO+0      ,3600 ,""    })});// Oct 6
         case 1998:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(60)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIXED(284)  ,UO+0      ,3600 ,""    })});// Oct 11
         case 1999:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(52)   ,UO-3600   ,0    ,""    }),  // Feb 21
                     ({FIXED(276)  ,UO+0      ,3600 ,""    })});// Oct 3
         case 2000:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(58)   ,UO-3600   ,0    ,""    }),  // Feb 27
                     ({LDAY (288,7),UO+0      ,3600 ,""    })});// Oct Sun>=8
         case 2001:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({LDAY (287,7),UO+0      ,3600 ,""    })});// Oct Sun>=8
         case 2002:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({FIXED(307)  ,UO+0      ,3600 ,""    })});// Nov 3
         case 2003:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({FIXED(292)  ,UO+0      ,3600 ,""    })});// Oct 19
         case 2004:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (53,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({FIXED(307)  ,UO+0      ,3600 ,""    })});// Nov 2
         case 2005:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({FIXED(289)  ,UO+0      ,3600 ,""    })});// Oct 16
         case 2006:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({FIXED(309)  ,UO+0      ,3600 ,""    })});// Nov 5
         case 2007:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(56)   ,UO-3600   ,0    ,""    }),  // Feb 25
                     ({LDAY (287,7),UO+0      ,3600 ,""    })});// Oct Sun>=8
         case 2012:
         case 2015:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(59,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=22
                     ({LDAYL(294,7),UO+0      ,3600 ,""    })});// Oct Sun>=15
         case 2008..2011:
         case 2013..2014:
         case 2016..2017:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({LDAYL(294,7),UO+0      ,3600 ,""    })});// Oct Sun>=15
         case 2018:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    }),  // Feb Sun>=15
                     ({LDAY (311,7),UO+0      ,3600 ,""    })});// Nov Sun>=1
         case 2019:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (52,7) ,UO-3600   ,0    ,""    })});// Feb Sun>=15
         default: // ..1930 and 2020..
         case 1934..1948:
         case 1954..1962:
         case 1969..1984:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Chile
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1927:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(244)  ,UO+0      ,3600 ,""    })});// Sep 1
         case 1928..1931:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIX_L(244)  ,UO+0      ,3600 ,""    })});// Sep 1
         case 1932:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(92)   ,UO-3600   ,0    ,""    })});// Apr 1
         default: // ..1926:
         case 1933..1967:
            return ({({0           ,0         ,0    ,""    })});
         case 1968:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(308)  ,14400     ,3600 ,""    })});// Nov 3
         case 1969:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(89)   ,10800     ,0    ,""    }),  // Mar 30
                     ({FIXED(327)  ,14400     ,3600 ,""    })});// Nov 23
         case 1970:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(88)   ,10800     ,0    ,""    }),  // Mar 29
                     ({LDAY (288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 1971:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(73)   ,10800     ,0    ,""    }),  // Mar 14
                     ({LDAY (288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 1973:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (74,7) ,10800     ,0    ,""    }),  // Mar Sun>=9
                     ({FIXED(273)  ,14400     ,3600 ,""    })});// Sep 30
         case 1987:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(102)  ,10800     ,0    ,""    }),  // Apr 12
                     ({LDAY (288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 1990:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (74,7) ,10800     ,0    ,""    }),  // Mar Sun>=9
                     ({FIXED(259)  ,14400     ,3600 ,""    })});// Sep 16
         case 1998:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (74,7) ,10800     ,0    ,""    }),  // Mar Sun>=9
                     ({FIXED(270)  ,14400     ,3600 ,""    })});// Sep 27
         case 1999:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(94)   ,10800     ,0    ,""    }),  // Apr 4
                     ({LDAY (288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 1997:
         case 2008:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(89)   ,10800     ,0    ,""    }),  // Mar 30
                     ({LDAYL(288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 1972:
         case 1974..1986:
         case 1988..1989:
         case 1991..1996:
         case 2000..2007:
         case 2009:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(74,7) ,10800     ,0    ,""    }),  // Mar Sun>=9
                     ({LDAYL(288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 2010:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (97,7) ,10800     ,0    ,""    }),  // Apr Sun>=1
                     ({LDAY (288,7),14400     ,3600 ,""    })});// Oct Sun>=9
         case 2011:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (128,7),10800     ,0    ,""    }),  // May Sun>=2
                     ({LDAY (234,7),14400     ,3600 ,""    })});// Aug Sun>=16
         case 2012..2014:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(119,7),10800     ,0    ,""    }),  // Apr Sun>=23
                     ({LDAYL(251,7),14400     ,3600 ,""    })});// Sep Sun>=2
         case 2015:
            return ({({0           ,0         ,3600 ,""    })});
         case 2016..2018:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(135,7),10800     ,0    ,""    }),  // May Sun>=9
                     ({LDAYL(227,7),14400     ,3600 ,""    })});// Aug Sun>=9
         case 2022:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (98,7) ,10800     ,0    ,""    }),  // Apr Sun>=2
                     ({LDAY (258,7),14400     ,3600 ,""    })});// Sep Sun>=9
         case 2019..2021:
         case 2023..:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(98,7) ,10800     ,0    ,""    }),  // Apr Sun>=2
                     ({LDAYL(251,7),14400     ,3600 ,""    })});// Sep Sun>=2
      }
   }
}

class CO
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1992:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(124)  ,UO+0      ,3600 ,""    })});// May 3
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(37)   ,UO+82800  ,0    ,""    })});// Feb 6
         default: // ..1991 and 1994..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Ecuador
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1992:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(333)  ,UO+0      ,3600 ,""    })});// Nov 28
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(36)   ,UO-3600   ,0    ,""    })});// Feb 5
         default: // ..1991 and 1994..
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Falk
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1939:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (84,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=19
                     ({FIXED(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1938:
         case 1940..1942:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(84,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=19
                     ({LDAYL(273,7),UO+0      ,3600 ,""    })});// Sep lastSun
         case 1943:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(1)    ,UO-3600   ,0    ,""    })});// Jan 1
         default: // ..1936:
         case 1944..1982:
            return ({({0           ,0         ,0    ,""    })});
         case 1937:
         case 1983:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (273,7),UO+0      ,3600 ,""    })});// Sep lastSun
         case 1984:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (121,7),UO-3600   ,0    ,""    }),  // Apr lastSun
                     ({FIXED(260)  ,UO+0      ,3600 ,""    })});// Sep 16
         case 1985:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (120,7),UO-3600   ,0    ,""    }),  // Apr lastSun
                     ({LDAY (258,7),UO+0      ,3600 ,""    })});// Sep Sun>=9
         case 1986..2000:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(112,7),UO-3600   ,0    ,""    }),  // Apr Sun>=16
                     ({LDAYL(258,7),UO+0      ,3600 ,""    })});// Sep Sun>=9
         case 2001..2010:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(111,7),UO+3600   ,0    ,""    }),  // Apr Sun>=15
                     ({LDAYL(250,7),UO+7200   ,3600 ,""    })});// Sep Sun>=1
         case 2011..:
            return ({({0           ,0         ,3600 ,""    })});
      }
   }
}

class Para
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         default: // ..1974:
            return ({({0           ,0         ,0    ,""    })});// ?
         case 1975:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(60)   ,UO+0      ,0    ,""    }),  // Mar 1
                     ({FIXED(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1976..1978:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(60)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIX_L(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1989:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIXED(295)  ,UO+0      ,3600 ,""    })});// Oct 22
         case 1979..1988:
         case 1990:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIX_L(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIX_L(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1991:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({FIXED(279)  ,UO+0      ,3600 ,""    })});// Oct 6
         case 1992:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(61)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({FIXED(279)  ,UO+0      ,3600 ,""    })});// Oct 5
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(90)   ,UO-3600   ,0    ,""    }),  // Mar 31
                     ({FIXED(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1994..1995:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (59,7) ,UO-3600   ,0    ,""    }),  // Feb lastSun
                     ({FIXED(274)  ,UO+0      ,3600 ,""    })});// Oct 1
         case 1996:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(61)   ,UO-3600   ,0    ,""    }),  // Mar 1
                     ({LDAY (281,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 1997:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (59,7) ,UO-3600   ,0    ,""    }),  // Feb lastSun
                     ({LDAY (280,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 1998..2001:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(66,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAYL(280,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 2002..2003:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (97,7) ,UO-3600   ,0    ,""    }),  // Apr Sun>=1
                     ({LDAY (250,7),UO+0      ,3600 ,""    })});// Sep Sun>=1
         case 2004:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (98,7) ,UO-3600   ,0    ,""    }),  // Apr Sun>=1
                     ({LDAY (295,7),UO+0      ,3600 ,""    })});// Oct Sun>=15
         case 2005..2009:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(73,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=8
                     ({LDAYL(294,7),UO+0      ,3600 ,""    })});// Oct Sun>=15
         case 2010..2012:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(104,7),UO-3600   ,0    ,""    }),  // Apr Sun>=8
                     ({LDAYL(280,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
         case 2013..:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(87,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=22
                     ({LDAYL(280,7),UO+0      ,3600 ,""    })});// Oct Sun>=1
      }
   }
}

class Peru
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1938:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(1)    ,UO+0      ,3600 ,""    }),  // Jan 1
                     ({FIXED(91)   ,UO-3600   ,0    ,""    }),  // Apr 1
                     ({LDAY (273,7),UO+0      ,3600 ,""    })});// Sep lastSun
         case 1939:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (89,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=24
                     ({LDAY (273,7),UO+0      ,3600 ,""    })});// Sep lastSun
         case 1940:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (90,7) ,UO-3600   ,0    ,""    })});// Mar Sun>=24
         case 1986..1987:
         case 1990:
         case 1994:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(1)    ,UO+0      ,3600 ,""    }),  // Jan 1
                     ({FIXED(91)   ,UO-3600   ,0    ,""    })});// Apr 1
         default: // ..1937 and 1995..
         case 1941..1985:
         case 1988..1989:
         case 1991..1993:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

class Uruguay
{
   inherit TZRules;
   protected array(array(string|int)) jd_year_periods(int jd)
   {
      [int y,int yjd,int leap]=gregorian_yjd(jd);
      switch (y)
      {
         case 1923:
            return ({({0           ,0         ,0    ,""    }),  // ?
                     ({FIXED(274)  ,UO+0      ,1800 ,""    })});// Oct 1
         case 1924..1925:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({FIX_L(91)   ,UO-1800   ,0    ,""    }),  // Apr 1
                     ({FIX_L(274)  ,UO+0      ,1800 ,""    })});// Oct 1
         case 1926:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({FIXED(91)   ,UO-1800   ,0    ,""    })});// Apr 1
         case 1933:
            return ({({0           ,0         ,0    ,""    }),  
                     ({LDAY (304,7),UO+0      ,1800 ,""    })});// Oct lastSun
         case 1934..1938:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAYL(90,6) ,UO+84600  ,0    ,""    }),  // Mar lastSat
                     ({LDAYL(304,7),UO+0      ,1800 ,""    })});// Oct lastSun
         case 1939:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (90,6) ,UO+84600  ,0    ,""    }),  // Mar lastSat
                     ({FIXED(274)  ,UO+0      ,1800 ,""    })});// Oct 1
         case 1940:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (91,6) ,UO+84600  ,0    ,""    }),  // Mar lastSat
                     ({FIXED(301)  ,UO+0      ,1800 ,""    })});// Oct 27
         case 1941:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({LDAY (90,6) ,UO+84600  ,0    ,""    }),  // Mar lastSat
                     ({FIXED(213)  ,UO+0      ,1800 ,""    })});// Aug 1
         case 1942:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({FIXED(348)  ,UO-1800   ,1800 ,""    })});// Dec 14
         case 1943:
            return ({({0           ,0         ,1800 ,""    }),  
                     ({FIXED(73)   ,UO-1800   ,0    ,""    })});// Mar 14
         case 1959:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(144)  ,UO+0      ,1800 ,""    }),  // May 24
                     ({FIXED(319)  ,UO-1800   ,0    ,""    })});// Nov 15
         case 1960:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(17)   ,UO+0      ,3600 ,""    }),  // Jan 17
                     ({FIXED(66)   ,UO-3600   ,0    ,""    })});// Mar 6
         case 1965:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(94)   ,UO+0      ,3600 ,""    }),  // Apr 4
                     ({FIXED(269)  ,UO-3600   ,0    ,""    })});// Sep 26
         case 1968:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(148)  ,UO+0      ,1800 ,""    }),  // May 27
                     ({FIXED(336)  ,UO-1800   ,0    ,""    })});// Dec 1
         case 1970:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(115)  ,UO+0      ,3600 ,""    }),  // Apr 25
                     ({FIXED(165)  ,UO-3600   ,0    ,""    })});// Jun 14
         case 1972:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(114)  ,UO+0      ,3600 ,""    }),  // Apr 23
                     ({FIXED(198)  ,UO-3600   ,0    ,""    })});// Jul 16
         case 1974:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(13)   ,UO+0      ,5400 ,""    }),  // Jan 13
                     ({FIXED(69)   ,UO-5400   ,1800 ,""    }),  // Mar 10
                     ({FIXED(244)  ,UO-1800   ,0    ,""    }),  // Sep 1
                     ({FIXED(356)  ,UO+0      ,3600 ,""    })});// Dec 22
         case 1975:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(89)   ,UO-3600   ,0    ,""    })});// Mar 30
         case 1976:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(354)  ,UO+0      ,3600 ,""    })});// Dec 19
         case 1977:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(65)   ,UO-3600   ,0    ,""    }),  // Mar 6
                     ({FIXED(338)  ,UO+0      ,3600 ,""    })});// Dec 4
         case 1978:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=1
                     ({FIXED(351)  ,UO+0      ,3600 ,""    })});// Dec 17
         case 1979:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=1
                     ({FIXED(119)  ,UO+0      ,3600 ,""    })});// Apr 29
         case 1980:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(76)   ,UO-3600   ,0    ,""    })});// Mar 16
         case 1987:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(348)  ,UO+0      ,3600 ,""    })});// Dec 14
         case 1988:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(59)   ,UO-3600   ,0    ,""    }),  // Feb 28
                     ({FIXED(346)  ,UO+0      ,3600 ,""    })});// Dec 11
         case 1989:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(64)   ,UO-3600   ,0    ,""    }),  // Mar 5
                     ({FIXED(302)  ,UO+0      ,3600 ,""    })});// Oct 29
         case 1990:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(56)   ,UO-3600   ,0    ,""    }),  // Feb 25
                     ({LDAY (300,7),UO+0      ,3600 ,""    })});// Oct Sun>=21
         case 1991:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (66,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=1
                     ({LDAY (300,7),UO+0      ,3600 ,""    })});// Oct Sun>=21
         case 1992:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (67,7) ,UO-3600   ,0    ,""    }),  // Mar Sun>=1
                     ({FIXED(292)  ,UO+0      ,3600 ,""    })});// Oct 18
         case 1993:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(59)   ,UO-3600   ,0    ,""    })});// Feb 28
         case 2004:
            return ({({0           ,0         ,0    ,""    }),  
                     ({FIXED(263)  ,UO+0      ,3600 ,""    })});// Sep 19
         case 2005:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({FIXED(86)   ,UO+3600   ,0    ,""    }),  // Mar 27
                     ({FIXED(282)  ,UO+7200   ,3600 ,""    })});// Oct 9
         case 2006..2014:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAYL(73,7) ,UO+3600   ,0    ,""    }),  // Mar Sun>=8
                     ({LDAYL(280,7),UO+7200   ,3600 ,""    })});// Oct Sun>=1
         case 2015:
            return ({({0           ,0         ,3600 ,""    }),  
                     ({LDAY (73,7) ,UO+3600   ,0    ,""    })});// Mar Sun>=8
         default: // ..1922 and 2016..
         case 1927..1932:
         case 1944..1958:
         case 1961..1964:
         case 1966..1967:
         case 1969:
         case 1971:
         case 1973:
         case 1981..1986:
         case 1994..2003:
            return ({({0           ,0         ,0    ,""    })});
      }
   }
}

